from insights.parsers.krb5kdc_log import KerberosKDCLog
from insights.tests import context_wrap

from datetime import datetime

KRB5KDC_LOG = """
Apr 01 03:36:01 ldap.example.com krb5kdc[24554](info): TGS_REQ (7 etypes {18 17 23 3 1 24 -135}) 10.250.2.47: UNKNOWN_SERVER: authtime 0,  opshzl@EXAMPLE.COM for cifs/secifs1.example.com@EXAMPLE.COM, Server not found in Kerberos database
Apr 01 03:36:04 ldap.example.com krb5kdc[24554](info): TGS_REQ (7 etypes {18 17 23 3 1 24 -135}) 10.250.18.199: UNKNOWN_SERVER: authtime 0,  pdtsro@EXAMPLE.COM for cifs/nyqrm4.example.com@EXAMPLE.COM, Server not found in Kerberos database
Apr 01 03:36:11 ldap.example.com krb5kdc[24554](info): AS_REQ (4 etypes {18 17 16 23}) 10.250.3.150: NEEDED_PREAUTH: sasher@EXAMPLE.COM for krbtgt/EXAMPLE.COM@EXAMPLE.COM, Additional pre-authentication required
Apr 01 03:36:11 ldap.example.com krb5kdc[24570](info): AS_REQ (4 etypes {18 17 16 23}) 10.250.3.150: ISSUE: authtime 1427873771, etypes {rep=18 tkt=18 ses=18}, sasher@EXAMPLE.COM for krbtgt/EXAMPLE.COM@EXAMPLE.COM
Apr 01 03:36:11 ldap.example.com krb5kdc[24569](info): TGS_REQ (4 etypes {18 17 16 23}) 10.250.3.150: ISSUE: authtime 1427873771, etypes {rep=18 tkt=18 ses=18}, sasher@EXAMPLE.COM for HTTP/sepdt138.example.com@EXAMPLE.COM
Apr 01 03:36:11 ldap.example.com krb5kdc[24569](info): AS_REQ (4 etypes {18 17 16 23}) 10.250.17.96: NEEDED_PREAUTH: niz@EXAMPLE.COM for krbtgt/EXAMPLE.COM@EXAMPLE.COM, Additional pre-authentication required
Apr 01 03:36:11 ldap.example.com krb5kdc[24549](info): AS_REQ (4 etypes {18 17 16 23}) 10.250.17.96: NEEDED_PREAUTH: niz@EXAMPLE.COM for krbtgt/EXAMPLE.COM@EXAMPLE.COM, Additional pre-authentication required
Apr 01 03:36:11 ldap.example.com krb5kdc[24546](info): AS_REQ (4 etypes {18 17 16 23}) 10.250.17.96: NEEDED_PREAUTH: niz@EXAMPLE.COM for krbtgt/EXAMPLE.COM@EXAMPLE.COM, Additional pre-authentication required
Apr 01 03:36:33 ldap.example.com krb5kdc[24556](info): preauth (encrypted_timestamp) verify failure: Decrypt integrity check failed
Apr 01 03:36:36 ldap.example.com krb5kdc[24568](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
Apr 01 03:38:34 ldap.example.com krb5kdc[24551](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
Apr 01 03:39:43 ldap.example.com krb5kdc[24549](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
Apr 01 03:39:43 ldap.example.com krb5kdc[24557](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
Apr 01 03:40:01 ldap.example.com krb5kdc[24570](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
Apr 01 03:40:15 ldap.example.com krb5kdc[24557](info): preauth (encrypted_timestamp) verify failure: Decrypt integrity check failed
Apr 01 03:40:18 ldap.example.com krb5kdc[24560](info): preauth (encrypted_timestamp) verify failure: Decrypt integrity check failed
Apr 01 03:40:20 ldap.example.com krb5kdc[24555](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
Apr 01 03:40:21 ldap.example.com krb5kdc[24560](info): preauth (encrypted_timestamp) verify failure: No matching key in entry
Apr 01 03:40:42 ldap.example.com krb5kdc[24573](info): preauth (encrypted_timestamp) verify failure: Decrypt integrity check failed
Apr 01 03:40:42 ldap.example.com krb5kdc[24568](info): preauth (encrypted_timestamp) verify failure: Decrypt integrity check failed
krb5kdc: setsockopt(7,IPV6_V6ONLY,1) worked
"""


def test_krb5kdc_log():
    log = KerberosKDCLog(context_wrap(KRB5KDC_LOG))
    assert len(log.lines) == 21
    decrypt_failed_logs = log.get('Decrypt integrity check failed')
    assert len(decrypt_failed_logs) == 5
    assert decrypt_failed_logs[0] == {
        'raw_message': 'Apr 01 03:36:33 ldap.example.com krb5kdc[24556](info): preauth (encrypted_timestamp) verify failure: Decrypt integrity check failed',
        'timestamp': 'Apr 01 03:36:33',
        'system': 'ldap.example.com',
        'service': 'krb5kdc',
        'pid': '24556',
        'level': 'info',
        'message': 'preauth (encrypted_timestamp) verify failure: Decrypt integrity check failed'
    }
    # Lines that don't parse still get found and just have the 'line' key
    assert len(list(log.get('setsockopt'))) == 1
    assert log.get('setsockopt')[0] == {
        'raw_message': 'krb5kdc: setsockopt(7,IPV6_V6ONLY,1) worked'
    }

    # test get_after, including continuation line
    assert len(list(log.get_after(datetime(2016, 4, 1, 3, 40, 0)))) == 8



"""
VmaRaEnabledS390x - file ``/sys/kernel/mm/swap/vma_ra_enabled``
===============================================================

Parser to parse the output of file ``/sys/kernel/mm/swap/vma_ra_enabled``
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.vma_ra_enabled)
class VmaRaEnabledS390x(Parser):
    """
    Base class to parse ``/sys/kernel/mm/swap/vma_ra_enabled`` file,
    the file content will be stored in a string.

    Sample output for file::

        True

    Examples:
        >>> type(vma)
        <class 'insights.parsers.vma_ra_enabled_s390x.VmaRaEnabledS390x'>
        >>> vma.ra_enabled
        True

    Attributes:
        ra_enabled (bool): The result parsed

    Raises:
        SkipComponent: When file content is empty

    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Input content is empty")

        if content[0] == 'True':
            self.ra_enabled = True
        else:
            self.ra_enabled = False



import doctest
from insights.tests import context_wrap
from insights.parsers.mlx4_port import Mlx4Port as Mlx4PortParser
from insights.combiners import mlx4_port
from insights.combiners.mlx4_port import Mlx4Port

MLX4_CONTENT_1 = """
ib
"""
MLX4_CONTENT_2 = """  line 1
  line 2"""

MLX4_PATH_1 = '/sys/bus/pci/devices/0000:0c:00.0/mlx4_port1'
MLX4_PATH_2 = '/sys/bus/pci/devices/0000:0c:00.0/mlx4_port2'


def test_mlx4_port():
    parser1 = Mlx4PortParser(context_wrap(MLX4_CONTENT_1, path=MLX4_PATH_1))
    parser2 = Mlx4PortParser(context_wrap(MLX4_CONTENT_2, path=MLX4_PATH_2))
    result = Mlx4Port([parser1, parser2])
    assert result is not None
    assert set(result.keys()) == set(['mlx4_port1', 'mlx4_port2'])
    assert result['mlx4_port1'] == ['ib']
    assert result['mlx4_port2'] == ['line 1', 'line 2']


def test_mlx4_docs():
    parser1 = Mlx4PortParser(context_wrap(MLX4_CONTENT_1, path=MLX4_PATH_1))
    parser2 = Mlx4PortParser(context_wrap(MLX4_CONTENT_2, path=MLX4_PATH_2))
    env = {'mlx4port': Mlx4Port([parser1, parser2])}
    failed, total = doctest.testmod(mlx4_port, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import ibm_proc
from insights.parsers.ibm_proc import IBMPpcLparCfg, IBMFirmwareLevel
from insights.tests import context_wrap


PROC_PPC_LPARCFG = """
serial_number=IBM,123456789
system_type=IBM,8247-22L
""".strip()

PROC_IBM_FWL = """
FW950.30 (VL950_092)\x00
""".strip()

PROC_IBM_FWL_NG = """
FW950.30 VL950_092\x00
""".strip()


def test_ibm_proc():
    results = IBMPpcLparCfg(context_wrap(PROC_PPC_LPARCFG))
    assert results['system_type'] == '8247-22L'
    assert results['serial_number'] == '123456789'

    results = IBMFirmwareLevel(context_wrap(PROC_IBM_FWL))
    assert results.firmware_level == 'VL950_092'


def test_ibm_proc_empty():
    with pytest.raises(SkipComponent):
        IBMPpcLparCfg(context_wrap(''))

    with pytest.raises(SkipComponent):
        IBMFirmwareLevel(context_wrap(''))

    with pytest.raises(SkipComponent):
        IBMFirmwareLevel(context_wrap(PROC_IBM_FWL_NG))


def test_ibm_proc_doc_examples():
    env = {
        "ibm_mtm": IBMPpcLparCfg(context_wrap(PROC_PPC_LPARCFG)),
        "ibm_fwl": IBMFirmwareLevel(context_wrap(PROC_IBM_FWL))
    }
    failed, total = doctest.testmod(ibm_proc, globs=env)
    assert failed == 0



"""
SambaConfig - file ``/etc/samba/smb.conf``
==========================================
"""
import re

from insights.core import IniConfigFile
from insights.core.filters import add_filter
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsr import iniparser
from insights.specs import Specs

add_filter(Specs.samba, ["["])

add_filter(Specs.testparm_s, ["["])
add_filter(Specs.testparm_s, ["Server role:"])

add_filter(Specs.testparm_v_s, ["["])
add_filter(Specs.testparm_v_s, ["Server role:"])


@parser(Specs.samba)
class SambaConfig(IniConfigFile):
    """
    This parser reads the Samba configuration file ``/etc/samba/smb.conf``, which
    is in standard .ini format, with a couple of notable features:

    Note: It is needed for better resolution descriptions when it is necessary to know what exactly
    is in the configuration file. For generic tasks use ``SambaConfigs`` or ``SambaConfigsAll``
    instead.

    * Samba ignores spaces at the start of options, which the ConfigParser class
      normally does not. This spacing is stripped by this parser.
    * Samba likewise ignores spaces in section heading names.
    * Samba allows the same section to be defined multiple times, with the
      options therein being merged as if they were one section.
    * Samba allows options to be declared before the first section marker.
      This parser puts these options in a `global` section.
    * Samba treats ';' as a comment prefix, similar to '#'.

    Sample configuration file::

        # This is the main Samba configuration file. You should read the
        # smb.conf(5) manual page in order to understand the options listed
        #...
        #======================= Global Settings =====================================

        [global]
            workgroup = MYGROUP
            server string = Samba Server Version %v
            max log size = 50

        [homes]
            comment = Home Directories
            browseable = no
            writable = yes
        ;   valid users = %S
        ;   valid users = MYDOMAIN\%S

        [printers]
            comment = All Printers
            path = /var/spool/samba
            browseable = no
            guest ok = no
            writable = no
            printable = yes

        # A publicly accessible directory, but read only, except for people in
        # the "staff" group
        [public]
           comment = Public Stuff
           path = /home/samba
           public = yes
           writable = yes
           printable = no
           write list = +staff

    Examples:
        >>> type(conf)
        <class 'insights.parsers.samba.SambaConfig'>
        >>> sorted(conf.sections()) == [u'global', u'homes', u'printers', u'public']
        True
        >>> global_options = conf.items('global')  # get a section as a dictionary
        >>> type(global_options) == type({})
        True
        >>> conf.get('public', 'comment') == u'Public Stuff'  # Accessor for section and option
        True
        >>> conf.getboolean('public', 'writable')  # Type conversion, but no default
        True
        >>> conf.getint('global', 'max log size')  # Same for integer conversion
        50
    """
    def parse_doc(self, content):
        # smb.conf is special from other ini files in the property that
        # whatever is before the first section (before the first section)
        # belongs to the [global] section. Therefore, the [global] section is
        # appended right at the beginning so that everything that would be
        # parsed as outside section belongs to [global].
        # Python 2.7 RawConfigParser automatically merges multiple instances
        # of the same section.  (And if that ever changes, test_samba.py will
        # catch it.)
        lstripped = ["[global]"] + [line.lstrip() for line in content]
        return iniparser.parse_doc("\n".join(lstripped), self, return_defaults=True, return_booleans=False)

    def parse_content(self, content):
        super(SambaConfig, self).parse_content(content)
        self._dict = dict()

        # Transform the section names so that whitespace around is stripped
        # and they are lowercase. smb.conf is special in the property that
        # section names and option names are case-insensitive and treated
        # like lower-case.
        for section in self.doc:
            section_dict = dict()
            _section = section.name.lower()

            for opt in section:
                options = []
                for o in section[opt.name]:
                    if o.value is not None:
                        options.append(str(o.value))
                    else:
                        options.append(o.value)

                section_dict[opt.name.lower()] = options[-1]

            if _section in self._dict:
                self._dict[_section].update(section_dict)
            else:
                self._dict[_section] = section_dict


@parser(Specs.testparm_s)
class SambaConfigs(SambaConfig):
    """
    This parser reads the Samba configuration from command `testparm -s` which is more reliable
    than parsing the config file, as it includes configuration in internal registry. It also
    includes server role.

    Note: This is the most suitable parser when only user changes to the configuration are important
    for the detection logic, i.e. misconfiguration.

    Attributes:
        server_role (string): Server role as reported by the command.
    """
    def parse_content(self, content):
        # Parse server role
        # The output of `testparm` sometimes includes progress lines such as
        # `Processing section "[homes]"`. These lines are output before the server
        # role definition, so only pass output from that line onward.
        server_role_line = None
        for i, line in enumerate(content):
            r = re.search(r"Server role:\s+(\S+)", line)
            if r:
                self.server_role = r.group(1)
                server_role_line = i
                break

        if server_role_line is None:
            raise ParseException("Server role not found.")

        super(SambaConfigs, self).parse_content(content[server_role_line:])


@parser(Specs.testparm_v_s)
class SambaConfigsAll(SambaConfigs):
    """
    This parser reads the Samba configuration from command `testparm -v -s` which is more reliable
    than parsing the config file, as it includes configuration in internal registry. It also
    includes all default values and server role.

    Note: This parser is needed for cases when active value of specific option is needed for the
    detection logic, irrespective of its origin from user changes or defaults, i.e. security
    vulnerabilities.

    Attributes:
        server_role (string): Server role as reported by the command.
    """
    pass



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import ipcs
from insights.parsers.ipcs import IpcsS, IpcsSI, IpcsM, IpcsMP
from insights.tests import context_wrap

IPCS_S = """
------ Semaphore Arrays --------
key        semid      owner      perms      nsems
0x00000000 557056     apache     600        1
0x00000000 589825     apache     600        1
0x00000000 131074     apache     600        1
0x0052e2c1 163843     postgres   600        17
0x0052e2c2 196612     postgres   600        17
0x0052e2c3 229381     postgres   600        17
0x0052e2c4 262150     postgres   600        17
0x0052e2c5 294919     postgres   600        17
0x0052e2c6 327688     postgres   600        17
0x0052e2c7 360457     postgres   600        17
0x00000000 622602     apache     600        1
0x00000000 655371     apache     600        1
0x00000000 688140     apache     600        1
""".strip()

IPCS_S_I = """
Semaphore Array semid=65536
uid=500  gid=501     cuid=500    cgid=501
mode=0600, access_perms=0600
nsems = 8
otime = Sun May 12 14:44:53 2013
ctime = Wed May  8 22:12:15 2013
semnum     value      ncount     zcount     pid
0          1          0          0          0
1          1          0          0          0
0          1          0          0          6151
3          1          0          0          2265
4          1          0          0          0
5          1          0          0          0
0          0          7          0          6152
7          1          0          0          4390
""".strip()

IPCS_M = """
------ Shared Memory Segments --------
key        shmid      owner      perms      bytes      nattch     status
0x0052e2c1 0          postgres   600        37879808   26
0x0052e2c2 1          postgres   600        41222144   24
""".strip()

IPCS_M_P = """
------ Shared Memory Creator/Last-op --------
shmid      owner      cpid       lpid
0          postgres   1833       23566
1          postgres   1105       9882
""".strip()

IPCS_M_AB = """
------ Shared Memory Segments --------
key        shid      owner      perms      bytes      nattch     status
0x0052e2c1 0          postgres   600        37879808   26
0x0052e2c2 1          postgres   600        41222144   24
""".strip()


def test_ipcs_s():
    sem = IpcsS(context_wrap(IPCS_S))

    assert '131074' in sem
    assert '11074' not in sem
    assert sem.get('262150') == {'owner': 'postgres', 'perms': '600',
                                 'nsems': '17', 'key': '0x0052e2c4'}
    assert sem['622602'] == {'owner': 'apache', 'perms': '600',
                             'nsems': '1', 'key': '0x00000000'}


def test_ipcs_si():
    sem = IpcsSI(context_wrap(IPCS_S_I))

    assert sem.semid == '65536'
    assert sem.pid_list == ['0', '2265', '4390', '6151', '6152']


def test_ipcs_m():
    shm = IpcsM(context_wrap(IPCS_M))
    assert '1' in shm
    assert '0' in shm
    assert '11074' not in shm
    assert shm.get('0').get('bytes') == '37879808'


def test_ipcs_mp():
    shm = IpcsMP(context_wrap(IPCS_M_P))
    assert '1' in shm
    assert '1074' not in shm
    assert shm.get('1').get('cpid') == '1105'


def test_ipcs_abnormal():
    with pytest.raises(SkipComponent) as pe:
        IpcsMP(context_wrap(""))
    assert "Nothing to parse." in str(pe)

    with pytest.raises(ParseException) as pe:
        IpcsMP(context_wrap(IPCS_M_AB))
    assert "Unexpected heading line." in str(pe)


def test_doc_examples():
    env = {
            'sem': IpcsS(context_wrap(IPCS_S)),
            'semi': IpcsSI(context_wrap(IPCS_S_I)),
            'shm': IpcsM(context_wrap(IPCS_M)),
            'shmp': IpcsMP(context_wrap(IPCS_M_P))
          }
    failed, total = doctest.testmod(ipcs, globs=env)
    assert failed == 0



# -*- coding: utf-8 -*-

import doctest

from insights.parsers import nss_rhel7
from insights.parsers.nss_rhel7 import NssRhel7
from insights.tests import context_wrap

NSS_RHEL7 = """
# To re-enable legacy algorithms, edit this file
# Note that the last empty line in this file must be preserved
library=
name=Policy
NSS=flags=policyOnly,moduleDB
config="disallow=MD5:RC4 allow=DH-MIN=1023:DSA-MIN=1023:RSA-MIN=1023:TLS-VERSION-MIN=tls1.0"
"""


def test_nss_rhel7():
    nss_rhel7 = NssRhel7(context_wrap(NSS_RHEL7))
    assert "config" in nss_rhel7
    assert "asdf" not in nss_rhel7
    assert (
        nss_rhel7.config == "disallow=MD5:RC4 allow=DH-MIN=1023:DSA-MIN=1023:RSA-MIN=1023:TLS-VERSION-MIN=tls1.0"
    )
    assert (
        nss_rhel7["config"] == "disallow=MD5:RC4 allow=DH-MIN=1023:DSA-MIN=1023:RSA-MIN=1023:TLS-VERSION-MIN=tls1.0"
    )
    assert (
        nss_rhel7.get("config") == "disallow=MD5:RC4 allow=DH-MIN=1023:DSA-MIN=1023:RSA-MIN=1023:TLS-VERSION-MIN=tls1.0"
    )
    assert nss_rhel7.get("asdf") is None


def test_doc_examples():
    env = {
        "nss_rhel7": NssRhel7(context_wrap(NSS_RHEL7)),
    }
    failed, total = doctest.testmod(nss_rhel7, globs=env)
    assert failed == 0



"""
EAP Json Report Files
=====================
EAP runtime will generate JSON report files under /var/tmp/insights-runtimes/uploads/

This module provides processing for the json report files of EAP runtimes.
"""

from insights.core import JSONParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.eap_json_reports)
class EAPJSONReports(JSONParser):
    """
    Class to parse the EAP json report files.

        Sample input data::

         {
           "version": "1.0.0",
           "idHash": "e38277334d0f6b6fdc6f3b831fb102cdd70f04faab5c38b0be36fb1aacb4236e",
           "basic": {
               "app.user.dir": "/opt/t_eap/jboss-eap-7.4",
               "java.specification.version": "1.8",
               "java.runtime.version": "1.8.0_362-b09",
               "java.class.path": "/opt/t_eap/jboss-eap-7.4/jboss-modules.jar",
               "system.os.version": "4.18.0-425.13.1.el8_7.x86_64",
               "jvm.args": [
                   "-D[Standalone]",
                   "-verbose:gc",
                   "-Xloggc:/opt/t_eap/jboss-eap-7.4/standalone/log/gc.log",
                   "-Djboss.modules.system.pkgs=org.jboss.byteman",
                   "-Dorg.jboss.boot.log.file=/opt/t_eap/jboss-eap-7.4/standalone/log/server.log",
                   "-Dlogging.configuration=file:/opt/t_eap/jboss-eap-7.4/standalone/configuration/logging.properties"
                 ]
                }
         }

    Examples:
        >>> type(eap_json_report)
        <class 'insights.parsers.eap_json_reports.EAPJSONReports'>
        >>> str(eap_json_report["version"])
        '1.0.0'
        >>> "idHash" in eap_json_report
        True
    """
    pass



"""
Simple language for defining predicates against a list or set of strings.

Operator Precedence:
    - ``!`` high - opposite truth value of its predicate
    - ``/`` high - starts a regex that continues until whitespace unless quoted
    - ``&`` medium - "and" of two predicates
    - ``|`` low - "or" of two predicates
    - ``,`` low - "or" of two predicates. Synonym for ``|``.

It supports grouping with parentheses and quoted strings/regexes surrounded
with either single or double quotes.

Examples:
    >>> pred = parse("a | b & !c")  # means (a or (b and (not c)))
    >>> pred(["a"])
    True
    >>> pred(["b"])
    True
    >>> pred(["b", "c"])
    False
    >>> pred = parse("/net | apache")
    >>> pred(["networking"])
    True
    >>> pred(["mynetwork"])
    True
    >>> pred(["apache"])
    True
    >>> pred(["security"])
    False
    >>> pred = parse("(a | b) & c")
    >>> pred(["a", "c"])
    True
    >>> pred(["b", "c"])
    True
    >>> pred(["a"])
    False
    >>> pred(["b"])
    False
    >>> pred(["c"])
    False

Regular expressions start with a forward slash ``/`` and continue until
whitespace unless they are quoted with either single or double quotes. This
means that they can consume what would normally be considered an operator or a
closing parenthesis if you aren't careful.

For example, this is a parse error because the regex consumes the comma:
    >>> pred = parse("/net, apache")
    Exception

Instead, do this:
    >>> pred = parse("/net , apache")

or this:
    >>> pred = parse("/net | apache")

or this:
    >>> pred = parse("'/net', apache")
"""
import re
import string
from insights.parsr import (Char, EOF, Forward, InSet, Many, Opt, String,
                            QuotedString, WS)


class Predicate(object):
    """
    Provides __call__ for invoking the Predicate like a function without having
    to explictly call its test method.
    """

    def __call__(self, value):
        return self.test(value)


class Eq(Predicate):
    """ The value must be in the set of values. """

    def __init__(self, value):
        self.value = value

    def test(self, values):
        return self.value in values


class Regex(Predicate):
    """ The regex must match at least one of the values. """

    def __init__(self, value):
        self.regex = re.compile(value)

    def test(self, values):
        return any(self.regex.search(v) for v in values)


class Not(Predicate):
    """ The values must not satisfy the wrapped condition. """

    def __init__(self, pred):
        self.pred = pred

    def test(self, value):
        return not self.pred.test(value)


class And(Predicate):
    """ The values must satisfy both the left and the right condition. """

    def __init__(self, left, right):
        self.left = left
        self.right = right

    def test(self, value):
        return self.left.test(value) and self.right.test(value)


class Or(Predicate):
    """ The values must satisfy either the left or the right condition. """

    def __init__(self, left, right):
        self.left = left
        self.right = right

    def test(self, value):
        return self.left.test(value) or self.right.test(value)


def negate(args):
    op, p = args
    return Not(p) if op else p


def oper(args):
    left, rest = args
    for op, right in rest:
        if op == "&":
            left = And(left, right)
        if op in ",|":
            left = Or(left, right)
    return left


expr = Forward()

bare = String(set(string.printable) - (set(string.whitespace) | set(")&,|")))

tag = (QuotedString | bare).map(Eq)

regex_body = QuotedString | String(set(string.printable) - set(string.whitespace))
regex = Char("/") >> regex_body.map(Regex)

factor_body = ((Char("(") >> expr << Char(")")) | regex | tag)
factor = (WS >> Opt(Char("!")) + factor_body << WS).map(negate)

term = (factor + Many(Char("&") + factor)).map(oper)
expr <= (term + Many(InSet(",|") + term)).map(oper)

parse = expr << EOF



"""
MultipathDevices - command ``multipath -v4 -ll``
================================================

This function converts the output of the ``multipath -v4 -ll`` command and
stores the data around each multipath device given.

Examples:
    >>> type(mpaths)
    <class 'insights.parsers.multipath_v4_ll.MultipathDevices'>
    >>> len(mpaths)  # Can treat the object as a list to iterate through
    3
    >>> mpaths[0]['alias']
    'mpathg'
    >>> mpaths[0]['size']
    '54T'
    >>> mpaths[0]['dm_name']
    'dm-2'
    >>> mpaths[0]['wwid']
    '36f01faf000da360b0000033c528fea6d'
    >>> groups = mpaths[0]['path_group']  # List of path groups for this device
    >>> groups[0]['status']
    'active'
    >>> len(groups[0]['path'])
    4
    >>> path0 = groups[0]['path'][0]  # Each path group has an array of paths
    >>> path0[1]  # Paths are stored as a list of items
    'sdc'
    >>> path0[-1]
    'running'
    >>> mpaths.dms  # List of device names found
    ['dm-2', 'dm-4', 'dm-5']
    >>> mpaths.by_dm['dm-2']['alias']  # Access by device name
    'mpathg'
    >>> mpaths.aliases  # Aliases found (again, in order)
    ['mpathg', 'mpathe']
    >>> mpaths.by_alias['mpathg']['dm_name']  # Access by alias
    'dm-2'
    >>> mpaths.by_wwid['36f01faf000da360b0000033c528fea6d']['dm_name']
    'dm-2'
"""

import re
import shlex
from insights import parser, CommandParser
from insights.specs import Specs


@parser(Specs.multipath__v4__ll)
class MultipathDevices(CommandParser):
    """
    ``multipath_-v4_ll`` command output

    Example input::

        ===== paths list =====
        uuid hcil    dev dev_t pri dm_st chk_st vend/prod/rev       dev_st
             0:0:0:0 sda 8:0   -1  undef ready  VMware,Virtual disk running
             3:0:0:1 sdb 8:16  -1  undef ready  IET,VIRTUAL-DISK    running
             4:0:0:1 sdc 8:32  -1  undef ready  IET,VIRTUAL-DISK    running
        Oct 28 14:02:44 | *word = 0, len = 1
        Oct 28 14:02:44 | *word = E, len = 1
        Oct 28 14:02:44 | *word = 1, len = 1
        Oct 28 14:02:44 | *word = 0, len = 1
        Oct 28 14:02:44 | *word = A, len = 1
        Oct 28 14:02:44 | *word = 0, len = 1
        mpathg (36f01faf000da360b0000033c528fea6d) dm-2 DELL,MD36xxi
        size=54T features='3 queue_if_no_path pg_init_retries 50' hwhandler='1 rdac' wp=rw
        |-+- policy='round-robin 0' prio=0 status=active
        | |- 12:0:0:1 sdc 8:32   active ready running
        | |- 11:0:0:1 sdi 8:128  active ready running
        | |- 15:0:0:1 sdo 8:224  active ready running
        | `- 17:0:0:1 sdv 65:80  active ready running
        `-+- policy='round-robin 0' prio=0 status=enabled
          |- 13:0:0:1 sdf 8:80   active ready running
          |- 14:0:0:1 sdl 8:176  active ready running
          |- 16:0:0:1 sdr 65:16  active ready running
          `- 18:0:0:1 sdx 65:112 active ready running
        mpathe (36f01faf000da3761000004323aa6fbce) dm-4 DELL,MD36xxi
        size=54T features='3 queue_if_no_path pg_init_retries 55' hwhandler='1 rdac' wp=rw
        |-+- policy='round-robin 0' prio=0 status=active
        | |- 13:0:0:2 sdg 8:96   active faulty running
        | |- 14:0:0:2 sdm 8:192  active faulty running
        | |- 16:0:0:2 sds 65:32  active faulty running
        | `- 18:0:0:2 sdy 65:128 active faulty running
        `-+- policy='round-robin 0' prio=0 status=enabled
          |- 12:0:0:2 sdd 8:48   active faulty running
          |- 11:0:0:2 sdj 8:144  active faulty running
          |- 15:0:0:2 sdp 8:240  active faulty running
          `- 17:0:0:2 sdw 65:96  active faulty running
        36001405b1629f80d52a4c898f8856e43 dm-5 LIO-ORG ,block0_sdb
        size=2.0G features='0' hwhandler='0' wp=rw
        |-+- policy='service-time 0' prio=1 status=active
        | `- 3:0:0:0 sdc 8:32 active ready running
        `-+- policy='service-time 0' prio=1 status=enabled
          `- 4:0:0:0 sdb 8:16 active ready running

    Example data structure produced::

        devices = [
          {
            "alias": "mpathg",
            "wwid": "36f01faf000da360b0000033c528fea6d",
            "dm_name": "dm-2",
            "venprod": "DELL,MD36xxi",
            "size": "54T",
            "features": "3 queue_if_no_path pg_init_retries 50",
            "hwhandler": "1 rdac",
            "wp": "rw",
            "path_group": [
                 {
                    "policy": "round-robin 0",
                    "prio": "0"
                    "status": "active"
                    "path": [
                        ['12:0:0:1', 'sdc', '8:32', 'active', 'ready', 'running'],
                        ['11:0:0:1', 'sdi', '8:128', 'active', 'ready', 'running'],
                        ['15:0:0:1', 'sdo', '8:224', 'active', 'ready', 'running'],
                        ['17:0:0:1', 'sdv', '65:80', 'active', 'ready', 'running']
                    ]
                 }, {
                    "policy": "round-robin 0",
                    "prio": "0"
                    "status": "enabled"
                    "path": [
                        ['13:0:0:1', 'sdf', '8:80', 'active', 'ready', 'running'],
                        ['14:0:0:1', 'sdl', '8:176', 'active', 'ready', 'running'],
                        ['16:0:0:1', 'sdr', '65:16', 'active', 'ready', 'running'],
                        ['18:0:0:1', 'sdx', '65:112','active', 'ready', 'running']
                    ]
                }
            ]
          },...
        ]

        raw_info_lines = [
            "===== paths list =====",
            "uuid hcil    dev dev_t pri dm_st chk_st vend/prod/rev       dev_st",
            "     0:0:0:0 sda 8:0   -1  undef ready  VMware,Virtual disk running",
            "     3:0:0:1 sdb 8:16  -1  undef ready  IET,VIRTUAL-DISK    running",
            "     4:0:0:1 sdc 8:32  -1  undef ready  IET,VIRTUAL-DISK    running",
            "Oct 28 14:02:44 | *word = 0, len = 1",
            ...
        ]

    Attributes:
        devices (list): List of devices found, in order
        dms (list): Device mapper names of each device, in order found
        aliases (list): Alias of each device, in order found
        wwids (list):  World Wide ID
        by_dm (dict): Access to each device by device mapper name
        by_alias (dict): Access to each device by alias
        by_wwid (dict): Access to each device by World Wide ID
        raw_info_lines(list): List of raw info lines found, in order
    """

    def parse_content(self, content):
        self.devices = []
        self.raw_info_lines = []

        mpath_dev_all = []
        mpath_dev = {}
        path_info = []
        path_group = []
        path_group_attr = {}

        MPATH_WWID_REG = re.compile(r'\(?([A-Za-z0-9_\s]+)\)?\s+dm-')
        PROPERTY_SQBRKT_REG = re.compile(r"\[(?P<key>\w+)=(?P<value>[^\]]+)\]")
        PATHGROUP_POLICY_STR = \
            r"(?:policy=')?(?P<policy>(?:round-robin|queue-length|service-time) \d)" + \
            r"(?:' | \[)prio=(?P<priority>\d+)(?:\]\[| status=)" + \
            r"(?P<status>\w+)(?:\]|)"
        PATHGROUP_POLICY_REG = re.compile(PATHGROUP_POLICY_STR)
        HCTL_REG = re.compile(r'(?:[`|]-(?:\+-)?|\\_) (\d+:){3}\d+')

        for line in content:
            m = MPATH_WWID_REG.search(line)

            if m:
                # Save previous path group info if we have any:
                # Now that we've got a valid path, append the group data if we
                # haven't already
                if path_info:
                    path_group_attr['path'] = path_info
                    path_group.append(path_group_attr)
                    # Must reset path group info to not carry on into new device
                    path_group_attr = {}
                    path_info = []
                    mpath_dev['path_group'] = path_group
                    mpath_dev_all.append(mpath_dev)

                mpath_dev = {}
                path_group = []
                wwid = m.group(1)
                no_alias = line.startswith(wwid)
                (dm, venprod) = re.findall(r".*(dm-\S+)\s+(.*)", line)[0]
                if not no_alias:
                    (dm, venprod) = re.findall(r"\w+\s+\(.*\)\s+(dm-\S+)\s+(.*)", line)[0]
                    mpath_dev['alias'] = line.split()[0]
                mpath_dev['wwid'] = wwid
                mpath_dev['dm_name'] = dm
                mpath_dev['venprod'] = venprod

            elif 'size=' in line:
                if '][' in line:
                    # Old RHEL 5 format:
                    for (k, v) in PROPERTY_SQBRKT_REG.findall(line):
                        mpath_dev[k] = v
                    # Handle un-named write policy attribute on the end:
                    mpath_dev['wp'] = line[-3:-1]
                else:
                    # Newer RHEL 6 format:
                    attr_line = shlex.split(line)
                    for item in attr_line:
                        (k, v) = item.split('=', 1)
                        mpath_dev[k] = v

            elif PATHGROUP_POLICY_REG.search(line):
                m = PATHGROUP_POLICY_REG.search(line)
                # New path info - save the previous if we have one:
                if path_info:
                    path_group_attr['path'] = path_info
                    path_group.append(path_group_attr)

                path_group_attr = {}
                path_info = []
                path_group_attr['policy'] = m.group('policy')
                path_group_attr['prio'] = m.group('priority')
                path_group_attr['status'] = m.group('status')

            elif HCTL_REG.search(line):
                colon_index = line.index(":")
                # Dodgy hack to convert RHEL 5 attributes in square brackets into
                # spaced out words to combine into the list
                line = line.replace('[', ' ').replace(']', ' ')
                path_info.append(line[colon_index - 2:].split())
            else:
                self.raw_info_lines.append(line)

        # final save of outstanding path and path group data:
        if path_info:
            path_group_attr['path'] = path_info
            path_group.append(path_group_attr)
        if path_group:
            mpath_dev['path_group'] = path_group
            mpath_dev_all.append(mpath_dev)

        self.devices = mpath_dev_all

        # Create some extra accessor properties
        self.dms = [path['dm_name'] for path in self.devices if 'dm_name' in path]
        self.by_dm = dict((path['dm_name'], path) for path in self.devices if 'dm_name' in path)
        self.aliases = [path['alias'] for path in self.devices if 'alias' in path]
        self.by_alias = dict((path['alias'], path) for path in self.devices if 'alias' in path)
        self.wwids = [path['wwid'] for path in self.devices if 'wwid' in path]
        self.by_wwid = dict((path['wwid'], path) for path in self.devices if 'wwid' in path)

    def __len__(self):
        """
        The length of the devices list
        """
        return len(self.devices)

    def __iter__(self):
        """
        Iterate through the devices list
        """
        for device in self.devices:
            yield device

    def __getitem__(self, idx):
        """
        Fetch a device by index in devices list
        """
        return self.devices[idx]



from insights.parsers import meminfo
from insights.tests import context_wrap

import pytest

"""
The following meminfo comes from a RHEL 7.1 box.  There are three values that
are currently accounted for in MemInfo that aren't in here:

- CmaTotal
- CmaFree
- DirectMap1G
"""

MEMINFO = """
MemTotal:        8009912 kB
MemFree:          538760 kB
MemAvailable:    6820236 kB
Buffers:          157048 kB
Cached:          4893932 kB
SwapCached:          120 kB
Active:          2841500 kB
Inactive:        2565560 kB
Active(anon):     311596 kB
Inactive(anon):   505800 kB
Active(file):    2529904 kB
Inactive(file):  2059760 kB
Unevictable:           0 kB
Mlocked:               0 kB
SwapTotal:       3145724 kB
SwapFree:        3136352 kB
Dirty:               140 kB
Writeback:             0 kB
AnonPages:        355484 kB
Mapped:            51988 kB
Shmem:            461316 kB
Slab:            1982652 kB
SReclaimable:    1945228 kB
SUnreclaim:        37424 kB
KernelStack:        3568 kB
PageTables:         8504 kB
NFS_Unstable:          0 kB
Bounce:                0 kB
WritebackTmp:          0 kB
CommitLimit:     7150680 kB
Committed_AS:    1218948 kB
VmallocTotal:   34359738367 kB
VmallocUsed:      122268 kB
VmallocChunk:   34359607064 kB
HardwareCorrupted:     0 kB
AnonHugePages:    135168 kB
HugePages_Total:       0
HugePages_Free:        0
HugePages_Rsvd:        0
HugePages_Surp:        0
Hugepagesize:       2048 kB
DirectMap4k:       71664 kB
DirectMap2M:     8316928 kB
""".strip()


def test_meminfo():
    values = []
    for l in MEMINFO.splitlines():
        values.append(l.split()[1].strip())
    m = meminfo.MemInfo(context_wrap(MEMINFO))
    actual = [
        m.total, m.free, m.available, m.buffers, m.cached, m.swap.cached,
        m.active, m.inactive, m.anon.active, m.anon.inactive, m.file.active,
        m.file.inactive, m.unevictable, m.mlocked, m.swap.total, m.swap.free,
        m.dirty, m.writeback, m.anon.pages, m.mapped, m.shmem, m.slab.total,
        m.slab.reclaimable, m.slab.unreclaimable, m.kernel_stack, m.page_tables,
        m.nfs_unstable, m.bounce, m.writeback_tmp, m.commit.limit, m.commit.total,
        m.vmalloc.total, m.vmalloc.used, m.vmalloc.chunk, m.corrupted,
        m.huge_pages.anon, m.huge_pages.total, m.huge_pages.free,
        m.huge_pages.reserved, m.huge_pages.surplus, m.huge_pages.size,
        m.direct_map.kb, m.direct_map.mb
    ]
    for i in range(len(actual)):
        assert isinstance(actual[i], int), "Line %d's value is not an int: %s" % (i, type(actual[i]))
        assert actual[i] == int(values[i]) * 1024, "Line %d failed to match" % i

    assert m.swap.used == (int(values[14]) - int(values[15]) - int(values[5])) * 1024
    assert m.used == (8009912 - 538760) * 1024


MEMINFO_SHORT = """
MemTotal:        8009912 kB
MemAvailable:    6820236 kB
"""


def test_meminfo_short():
    # Test the various calculated properties when little or no data is
    # available.
    m = meminfo.MemInfo(context_wrap(MEMINFO_SHORT))
    with pytest.raises(TypeError):
        assert m.swap.used is None
    with pytest.raises(TypeError):
        assert m.used is None


def test_using_huge_pages():
    t = [
        "AnonHugePages:    135168 kB",
        "HugePages_Total:       0"
    ]
    m = meminfo.MemInfo(context_wrap(t))
    assert not m.huge_pages.using
    assert m.huge_pages.using_transparent
    t = [
        "AnonHugePages:    0 kB",
        "HugePages_Total:  123456",
        "HugePages_Rsvd:     4096",
    ]
    m = meminfo.MemInfo(context_wrap(t))
    assert m.huge_pages.using
    assert not m.huge_pages.using_transparent
    assert m.huge_pages.total == 123456
    assert m.huge_pages.reserved == 4096



"""
dmsetup commands - Command ``dmsetup``
======================================

Parsers for parsing and extracting data from output of commands related to
``dmsetup``.
Parsers contained in this module are:

DmsetupInfo - command ``dmsetup info -C``
-----------------------------------------

DmsetupStatus - command ``dmsetup status``
------------------------------------------

"""
from collections import namedtuple

from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import parse_delimited_table
from insights.specs import Specs


SetupInfo = namedtuple('SetupInfo', [
    'name', 'major', 'minor', 'open', 'segments', 'events',
    'live_table', 'inactive_table', 'suspended', 'readonly', 'uuid']
)
""" Data structure to represent dmsetup information """

SetupStatus = namedtuple('SetupStatus', [
    'device_name', 'start_sector', 'num_sectors', 'target_type',
    'target_args', 'parsed_args']
)
""" Data structure to represent dmsetup status """


@parser(Specs.dmsetup_info)
class DmsetupInfo(CommandParser):
    """
    ``dmsetup info -C`` command output

    Example input::

        Name               Maj Min Stat Open Targ Event  UUID
        VG00-tmp           253   8 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4
        VG00-home          253   3 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxCqXOnbGe2zjhX923dFiIdl1oi7mO9tXp
        VG00-var           253   6 L--w    1    2      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxicvyvt67113nTb8vMlGfgdEjDx0LKT2O
        VG00-swap          253   1 L--w    2    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax3Ll2XhOYZkylx1CjOQi7G4yHgrIOsyqG
        VG00-root          253   0 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxKpnAKYhrYMYMNMwjegkW965bUgtJFTRY
        VG00-var_log_audit 253   5 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxwQ8R0XWJRm86QX3befq1cHRy47Von6ZW

    Example data structure produced::

        data = [
          {
            'Stat': 'L--w',
            'Name': 'VG00-tmp',
            'Min': '8',
            'Targ': '1',
            'Maj': '253',
            'Open': '1',
            'Event': '0',
            'UUID': 'LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4'
          },...
        ]

    Attributes:
        data (list): List of devices found, in order
        names (list): Device names, in order found
        uuids (list): UUID
        by_name (dict): Access to each device by devicename
        by_uuid (dict): Access to each device by uuid
        info (list): List of devices found, in order using SetupInfo structure

    Example:
        >>> len(setup_info)
        6
        >>> setup_info.names[0]
        'VG00-tmp'
        >>> setup_info[1]['Maj']
        '253'
        >>> setup_info[1]['Stat']
        'L--w'
        >>> setup_info.info[-1].name
        'VG00-var_log_audit'
        >>> setup_info.info[-1].major
        253
        >>> setup_info.info[-1].live_table
        True
        >>> setup_info.info[-1].readonly
        False
    """

    def parse_content(self, content):
        self.data = parse_delimited_table(content)

        self.names = [dm['Name'] for dm in self.data if 'Name' in dm]
        self.by_name = dict((dm['Name'], dm) for dm in self.data if 'Name' in dm)
        self.uuids = [dm['UUID'] for dm in self.data if 'UUID' in dm]
        self.by_uuid = dict((dm['UUID'], dm) for dm in self.data if 'UUID' in dm)
        self.info = []
        for dm in self.data:
            self.info.append(SetupInfo(
                name=dm.get('Name'),
                major=int(dm.get('Maj')) if 'Maj' in dm and dm.get('Maj').isdigit() else None,
                minor=int(dm.get('Min')) if 'Min' in dm and dm.get('Min').isdigit() else None,
                open=int(dm.get('Open')) if 'Open' in dm and dm.get('Open').isdigit() else None,
                segments=int(dm.get('Targ')) if 'Targ' in dm and dm.get('Targ').isdigit() else None,
                events=int(dm.get('Event')) if 'Event' in dm and dm.get('Event').isdigit() else None,
                live_table=dm.get('Stat', '----')[0] == 'L',
                inactive_table=dm.get('Stat', '----')[1] == 'I',
                suspended=dm.get('Stat', '----')[2] == 's',
                readonly=dm.get('Stat', '----')[3] == 'r',
                uuid=dm.get('UUID')
            ))

    def __len__(self):
        """
        The length of the devices list
        """
        return len(self.data)

    def __iter__(self):
        """
        Iterate through the devices list
        """
        for dm in self.data:
            yield dm

    def __getitem__(self, idx):
        """
        Fetch a device by index in devices list
        """
        return self.data[idx]


@parser(Specs.dmsetup_status)
class DmsetupStatus(CommandParser, list):
    """
    ``dmsetup status -C`` command output

    Example input::

        rootvg-tanlv: 0 6291456 linear
        rootvg-ssnap: 0 16384000 snapshot 1560768/5120000 6088
        rootvg-optvolapp: 0 8192000 snapshot-origin
        docker-253:10-1234567-0df13579: 0 20971520 thin 1922048 20971519
        docker-253:10-4254621-0496628a: 0 20971520 thin 1951744 20971519
        docker-253:10-4254621-d392682f: 0 20971520 thin 7106560 20971519
        rootvg-docker--pool: 0 129548288 thin-pool 1 20/49152 38/126512 - rw no_discard_passdown queue_if_no_space -
        rootvg-tmpvol: 0 2048000 linear
        rootvg-varvol: 0 18874368 snapshot Invalid
        rootvg-optvol: 0 8192000 snapshot 616408/5120000 2408
        rootvg-varvol-cow: 0 5120000 linear
        appsvg-lvapps_docker: 0 104857600 thin-pool 441 697/2048 20663/102400 - rw no_discard_passdown queue_if_no_space -

    Example data structure produced::

        [
            SetupStatus(
                device_name='rootvg-tanlv', start_sector='0',
                num_sectors='6291456', target_type='linear',
                target_args=None, parsed_args=None,
            ), ...
        ]

    Attributes:
        names (list): Device names, in order found
        by_name (dict): Access to each device by devicename
        unparseable_lines (list): Unparseable raw lines

    Example:
        >>> len(dmsetup_status)
        12
        >>> dmsetup_status.names[0]
        'rootvg-tanlv'
        >>> dmsetup_status[1].target_type
        'snapshot'
        >>> dmsetup_status[1].start_sector
        '0'
        >>> len(dmsetup_status.by_name)
        12
        >>> dmsetup_status[-1].parsed_args['used_metadata_blocks']
        '697'
        >>> dmsetup_status[-1].parsed_args['total_metadata_blocks']
        '2048'
        >>> dmsetup_status[-1].parsed_args['opts']
        ['rw', 'no_discard_passdown', 'queue_if_no_space', '-']
    """

    def parse_content(self, content):
        self.unparseable_lines = []
        if content[0].lower() == "no devices found":
            return

        for line in content:
            _device_name, _device_info_str = line.rsplit(':', 1)
            device_name = _device_name.strip()
            device_info_spl = _device_info_str.strip().split(' ', 3)
            if len(device_info_spl) < 3:
                self.unparseable_lines.append(line)
                continue
            target_type = device_info_spl[2]
            target_args = device_info_spl[3] if len(device_info_spl) == 4 else None
            parsed_args = None
            if target_args:
                try:
                    parsed_args = self._parse_target_args(target_type, target_args)
                except ParseException:
                    self.unparseable_lines.append(line)
            self.append(SetupStatus(
                device_name=device_name,
                start_sector=device_info_spl[0],
                num_sectors=device_info_spl[1],
                target_type=target_type,
                target_args=target_args,
                parsed_args=parsed_args,
            ))

    @property
    def names(self):
        return [dm[0] for dm in self]

    @property
    def by_name(self):
        return dict((dm[0], dm) for dm in self)

    def _parse_target_args(self, target_type, target_args):
        pars_func_name = '_parse_target_args_' + target_type.replace('-', '_')
        pars_func = getattr(self, pars_func_name, None)
        return pars_func(target_args) if pars_func else None

    def _parse_target_args_thin_pool(self, target_args):
        """
        Format:
            <transaction_id> <used_metadata_blocks>/<total_metadata_blocks>
            <used_data_blocks>/<total_data_blocks> <held_metadata_root>
            ro|rw|out_of_data_space [no_]discard_passdown [error|queue]_if_no_space
            needs_check|- metadata_low_watermark
        Refer to https://www.kernel.org/doc/Documentation/device-mapper/thin-provisioning.txt .
        """
        args = target_args.split()
        if len(args) < 8 or '/' not in args[1] or '/' not in args[2]:
            raise ParseException("Invalid thin_pool target_args: {0}".format(target_args))
        parsed_args = {}
        parsed_args['transaction_id'] = args[0]
        parsed_args['used_metadata_blocks'], parsed_args['total_metadata_blocks'] = args[1].split('/', 1)
        parsed_args['used_data_blocks'], parsed_args['total_data_blocks'] = args[2].split('/', 1)
        parsed_args['held_metadata_root'] = args[3]
        parsed_args['opts'] = args[4:8]
        parsed_args['metadata_low_watermark'] = args[8] if len(args) > 8 else None
        return parsed_args

    def _parse_target_args_thin(self, target_args):
        """
        Format:
            <nr_mapped_sectors> <highest_mapped_sector>
        Refer to https://www.kernel.org/doc/Documentation/device-mapper/thin-provisioning.txt .
        """
        args = target_args.split()
        if len(args) < 2:
            raise ParseException("Invalid thin target_args: {0}".format(target_args))
        parsed_args = {}
        parsed_args['nr_mapped_sectors'] = args[0]
        parsed_args['highest_mapped_sector'] = args[1]
        return parsed_args

    def _parse_target_args_snapshot(self, target_args):
        """
        Format:
            <sectors_allocated>/<total_sectors> <metadata_sectors>
        Refer to https://www.kernel.org/doc/Documentation/device-mapper/snapshot.txt .
        """
        args = target_args.split()
        if len(args) < 2 or '/' not in args[0]:
            raise ParseException("Invalid snapshot target_args: {0}".format(target_args))
        parsed_args = {}
        parsed_args['sectors_allocated'], parsed_args['total_sectors'] = args[0].split('/', 1)
        parsed_args['metadata_sectors'] = args[1]
        return parsed_args



"""
CUPS configuration files
========================

Parsers provided by this module are:

CupsdConf - file ``/etc/cups/cupsd.conf``
-----------------------------------------
CupsFilesConf - file ``/etc/cups/cups-files.conf``
--------------------------------------------------
"""

from insights.core import Parser, ConfigParser
from insights.core.plugins import parser
from insights.specs import Specs
from insights.parsers import get_active_lines
from insights.parsers.httpd_conf import DocParser
from insights.core.exceptions import ParseException
from insights.parsr.query import Entry


class CupsdConfBase(DocParser):

    def __call__(self, content):
        try:
            return self.Top(content)
        except Exception:
            raise ParseException("There was an exception when parsing the cupsd.conf file")


@parser(Specs.cupsd_conf)
class CupsdConf(ConfigParser):
    """
    Class for parsing the file ``/etc/cups/cupsd.conf``

    Sample file content::

        MaxLogSize 0
        LogLevel warn
        Listen localhost:631
        Listen /var/run/cups/cups.sock
        Browsing On
        BrowseLocalProtocols dnssd

        <Location />
          Order allow,deny
        </Location>

        <Policy default>
          JobPrivateAccess default
          JobPrivateValues default
          SubscriptionPrivateAccess default
          SubscriptionPrivateValues default

          <Limit Send-Document Send-URI Hold-Job Release-Job Restart-Job Purge-Jobs Set-Job-Attributes Create-Job-Subscription Renew-Subscription Cancel-Subscription Get-Notifications Reprocess-Job Cancel-Current-Job Suspend-Current-Job Resume-Job Cancel-My-Jobs Close-Job CUPS-Move-Job CUPS-Get-Document>
            Require user @OWNER @SYSTEM
            Order deny,allow
          </Limit>

          <Limit All>
            Order deny,allow
          </Limit>
        </Policy>

    Examples:
        >>> type(cupsd_conf)
        <class 'insights.parsers.cups_confs.CupsdConf'>
        >>> cupsd_conf['MaxLogSize'][-1].value
        0
        >>> cupsd_conf['Listen'][0].value
        'localhost:631'
        >>> cupsd_conf['Listen'][-1].value
        '/var/run/cups/cups.sock'
        >>> cupsd_conf[('Location', '/')]['Order'][-1].value
        'allow,deny'
        >>> cupsd_conf[('Policy','default')][('Limit','Send-Document')]['Order'][0].value
        'deny,allow'
        >>> 'JobPrivateAccess' in cupsd_conf[('Policy','default')]
        True
    """

    def __init__(self, *args, **kwargs):
        self.parse = CupsdConfBase(self)
        super(CupsdConf, self).__init__(*args, **kwargs)

    def parse_doc(self, content):
        if isinstance(content, list):
            content = "\n".join(content)
        result = self.parse(content)[0]
        return Entry(children=result, src=self)


@parser(Specs.cups_files_conf)
class CupsFilesConf(Parser, dict):
    """
    Class for parsing the file ``/etc/cups/cups-files.conf``

    Sample file content::

        FatalErrors config
        SyncOnClose Yes
        SystemGroup sys root wheel
        AccessLog syslog
        ErrorLog syslog
        PageLog syslog

    Examples:
        >>> type(cups_files_conf)
        <class 'insights.parsers.cups_confs.CupsFilesConf'>
        >>> cups_files_conf['FatalErrors']
        'config'
        >>> cups_files_conf['SyncOnClose']
        'Yes'
        >>> 'wheel' in cups_files_conf['SystemGroup']
        True
    """

    def parse_content(self, content):
        if not content:
            raise ParseException('Empty Content')

        for line in get_active_lines(content):
            k, v = [i.strip() for i in line.split(None, 1)]
            if k not in self:
                self[k] = v if len(v.split()) == 1 else v.split()
            else:
                _v = self[k]
                _v = [_v] if not isinstance(_v, list) else _v
                if v not in _v:
                    _v.append(v)
                    self[k] = _v



"""
LsUsrLib64 - command ``ls -lan /usr/lib64``
===========================================

The ``ls -lan /usr/lib64`` command provides information for the listing of the
``/usr/lib64`` directory.

See :class:`insights.core.FileListing` class for additional information.

Sample directory list::

    total 447460
    dr-xr-xr-x. 150 0 0    77824 Jul 30 16:39 .
    drwxr-xr-x.  13 0 0     4096 Apr 30  2017 ..
    drwxr-xr-x.   3 0 0       20 Nov  3  2016 krb5
    -rwxr-xr-x.   1 0 0   155464 Oct 28  2016 ld-2.17.so
    drwxr-xr-x.   3 0 0       20 Jun 10  2016 ldb
    lrwxrwxrwx.   1 0 0       10 Apr 30  2017 ld-linux-x86-64.so.2 -> ld-2.17.so
    lrwxrwxrwx.   1 0 0       21 Apr 30  2017 libabrt_dbus.so.0 -> libabrt_dbus.so.0.0.1

Examples:

    >>> "krb5" in ls_usr_lib64
    False
    >>> "/usr/lib64" in ls_usr_lib64
    True
    >>> "krb5" in ls_usr_lib64.dirs_of('/usr/lib64')
    True
    >>> ls_usr_lib64.dir_entry('/usr/lib64', 'ld-linux-x86-64.so.2')['type']
    'l'
"""
from insights import parser, CommandParser, FileListing
from insights.specs import Specs
from insights.core.filters import add_filter
from insights.util import deprecated


add_filter(Specs.ls_usr_lib64, "total")


@parser(Specs.ls_usr_lib64)
class LsUsrLib64(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlanFiltered` instead.

    Parses output of ``ls -lan /usr/lib64`` command.
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsUsrLib64, "Please use the :class:`insights.parsers.ls.LSlanFiltered` instead.", "3.5.0")
        super(LsUsrLib64, self).__init__(*args, **kwargs)



"""
RosConfig - file ``/var/lib/pcp/config/pmlogger/config.ros``
============================================================
This class provides parsing for the files:
    ``/var/lib/pcp/config/pmlogger/config.ros``
"""

from insights import parser, Parser
from insights.specs import Specs
from insights.parsr import EOF, EOL, Char, Literal, Many, OneLineComment, Opt, QuotedString, String, WSChar
import string


# https://man7.org/linux/man-pages/man1/pmlogger.1.html#CONFIGURATION_FILE_SYNTAX

WS = Many(WSChar | EOL | OneLineComment("#"))

Log = WS >> Literal("log") << WS
MandatoryOn = WS >> Literal("mandatory on") << WS
MandatoryOff = WS >> Literal("mandatory off") << WS
MandatoryMaybe = WS >> Literal("mandatory maybe") << WS
AdvisoryOn = WS >> Literal("advisory on") << WS
AdvisoryOff = WS >> Literal("advisory off") << WS

Once = WS >> Literal("once") << WS
Default = WS >> Literal("default") << WS
Every = WS >> Literal("every") << WS
UnsignedInt = String(string.digits).map(int)
TimeUnits = WS >> String(string.ascii_letters) << WS
Freq = Opt(Every) >> (UnsignedInt + TimeUnits)

Interval = Once | Default | Freq
OnStates = MandatoryOn | AdvisoryOn
OtherStates = MandatoryMaybe | MandatoryOff | AdvisoryOff

Preamble = Opt(Log) >> ((OnStates + Interval) | OtherStates)

LeftBrace = WS >> Char("{") << WS
RightBrace = WS >> Char("}") << WS
Comma = WS >> Char(",") << WS

Name = WS >> String(string.ascii_letters + string.digits + "-._") << WS

LeftBracket = WS >> Char('[') << WS
RightBracket = WS >> Char(']') << WS
InstanceName = QuotedString | UnsignedInt | Name
InstanceNames = LeftBracket >> InstanceName.sep_by(Comma | WS) << RightBracket
MetricSpec = Name + Opt(InstanceNames, default=[])

OneMetricSpec = MetricSpec.map(lambda s: [s])
MultipleMetricSpecs = LeftBrace >> MetricSpec.sep_by(Comma | WS) << RightBrace
MetricSpecs = (OneMetricSpec | MultipleMetricSpecs).map(dict)

LogSpec = Preamble + MetricSpecs

LogSpecs = Many(LogSpec)

AccessHeader = WS >> Literal("[access]") << WS
Allow = WS >> Literal("allow") << WS
Disallow = WS >> Literal("disallow") << WS
AllowDisallow = Allow | Disallow


EnquireOp = WS >> Literal("enquire") << WS
AdvisoryOp = WS >> Literal("advisory") << WS
MandatoryOp = WS >> Literal("mandatory") << WS
AllExceptOp = WS >> Literal("all except") << WS
AllOp = WS >> Literal("all") << WS

Colon = WS >> Literal(":") << WS

Host = String(string.ascii_letters + string.digits + ".*:\"")
HostList = WS >> Host.sep_by(Comma) << WS

Operation = EnquireOp | AdvisoryOp | MandatoryOp | AllExceptOp | AllOp
OperationList = WS >> Operation.sep_by(Comma | WS) << WS

Semicolon = WS >> Char(";") << WS

AccessRule = AllowDisallow + HostList + Colon + OperationList << Semicolon
AccessRules = Many(AccessRule)
AccessControl = AccessHeader + AccessRules
Doc = LogSpecs + Opt(AccessControl)
parse = Doc << EOF


@parser(Specs.ros_config)
class RosConfig(Parser):
    """
    Sample input data is in the format::

        log mandatory on default {
            mem.util.used
            mem.physmem
            kernel.all.cpu.user
            kernel.all.cpu.sys
            kernel.all.cpu.nice
            kernel.all.cpu.steal
            kernel.all.cpu.idle
            kernel.all.cpu.wait.total
            disk.all.total
            mem.util.cached
            mem.util.bufmem
            mem.util.free
        }
        [access]
        disallow .* : all;
        disallow :* : all;
        allow local:* : enquire;

    Examples:
        >>> type(ros_input)
        <class 'insights.parsers.ros_config.RosConfig'>
        >>> ros_input.rules[0]['allow_disallow']
        'disallow'
        >>> ros_input.rules[0]['hostlist']
        ['.*']
        >>> ros_input.rules[0]['operationlist']
        ['all']
        >>> ros_input.specs[0].get('state')
        'mandatory on'
        >>> ros_input.specs[0].get('metrics')['mem.util.used']
        []
        >>> ros_input.specs[0].get('metrics')['kernel.all.cpu.user']
        []
        >>> ros_input.specs[0].get('logging_interval')
        'default'

    Attributes:
        data(list): All parsed options and log files are stored in this
            list.
        specs(list of dicts): List of the ROS specifications present in
            config.ros file.
        rules(list of dicts): List of access control rules applied for
            config.ros file.

    """
    def parse_content(self, content):
        self.data = parse("\n".join(content))
        self.specs = []
        specifications = self.data[0]
        for spec in specifications:
            state = spec[0][0]
            logging_interval = spec[0][1] if state.endswith('on') else None
            self.specs.append({'state': state, 'logging_interval': logging_interval, 'metrics': spec[1]})
        access_rules = self.data[1][1]
        self.rules = []
        for rule in access_rules:
            self.rules.append({'allow_disallow': rule[0], 'hostlist': rule[1], 'operationlist': rule[3]})



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import sap_dev_trace_files
from insights.parsers.sap_dev_trace_files import SapDevDisp, SapDevRd
from insights.tests import context_wrap

SAP_DEV_DISP = """
---------------------------------------------------
trc file: "dev_disp", trc level: 1, release: "745"
---------------------------------------------------

*** WARNING => DpHdlDeadWp: wp_adm slot for W7 has no pid [dpxxwp.c     1353]
DpSkipSnapshot: last snapshot created at Sun Aug 18 17:15:25 2019, skip new snapshot
*** WARNING => DpHdlDeadWp: wp_adm slot for W8 has no pid [dpxxwp.c     1353]
DpSkipSnapshot: last snapshot created at Sun Aug 18 17:15:25 2019, skip new snapshot
*** WARNING => DpHdlDeadWp: wp_adm slot for W9 has no pid [dpxxwp.c     1353]

Sun Aug 18 17:17:45 2019
DpSkipSnapshot: last snapshot created at Sun Aug 18 17:17:45 2019, skip new snapshot
DpCheckSapcontrolProcess: sapcontrol with pid 1479 terminated
*** WARNING => DpRequestProcessingCheck: potential request processing problem detected (14. check) [dpxxwp.c     4633]
""".strip()

SAP_DEV_RD = """
---------------------------------------------------
trc file: "dev_rd", trc level: 1, release: "745"
---------------------------------------------------
Thu Jul 18 02:59:37 2019
gateway (version=745.2015.12.21 (with SSL support))
gw/reg_no_conn_info = 1
* SWITCH TRC-RESOLUTION from 1 to 1
gw/sim_mode : set to 0
gw/logging : ACTION=Ss LOGFILE=gw_log-%y-%m-%d SWITCHTF=day MAXSIZEKB=100
NI buffering enabled
CCMS: initialize CCMS Monitoring for ABAP instance with J2EE addin.

Thu Jul 18 02:59:38 2019
CCMS: Initialized monitoring segment of size 60000000.
CCMS: Initialized CCMS Headers in the shared monitoring segment.
CCMS: Checking Downtime Configuration of Monitoring Segment.

Thu Jul 18 02:59:39 2019
CCMS: AlMsUpload called by wp 1002.
CCMS: AlMsUpload successful for /usr/sap/RH1/D00/log/ALMTTREE (542 MTEs).

Thu Jul 18 02:59:40 2019
GwIInitSecInfo: secinfo version = 2
GwIRegInitRegInfo: reg_info file /usr/sap/RH1/D00/data/reginfo not found
""".strip()


SapDevDisp.keep_scan('warning_lines', "WARNING")
SapDevRd.keep_scan('ccms', "CCMS:")
DISP_PATH = '/usr/sap/RH1/D00/work/dev_disp'
RD_PATH = '/usr/sap/RH2/D03/work/dev_rd'


def test_dev_disp():
    dev_disp = SapDevDisp(context_wrap(SAP_DEV_DISP, path=DISP_PATH))
    assert len(dev_disp.warning_lines) == len(dev_disp.get("WARNING"))
    assert dev_disp.sid == DISP_PATH.split('/')[3]
    assert dev_disp.instance == DISP_PATH.split('/')[4]
    with pytest.raises(ParseException):
        dev_disp.get_after()


def test_dev_rd():
    dev_rd = SapDevRd(context_wrap(SAP_DEV_RD, path=RD_PATH))
    assert len(dev_rd.ccms) == len(dev_rd.get("CCMS:"))
    assert dev_rd.sid == RD_PATH.split('/')[3]
    assert dev_rd.instance == RD_PATH.split('/')[4]
    with pytest.raises(ParseException):
        dev_rd.get_after()


def test_dev_docs():
    env = {
        "dev_disp": SapDevDisp(context_wrap(SAP_DEV_DISP, path=DISP_PATH)),
        "dev_rd": SapDevRd(context_wrap(SAP_DEV_RD, path=RD_PATH))
    }
    failed, total = doctest.testmod(sap_dev_trace_files, globs=env)
    assert failed == 0



"""
GRUB environment block
======================
This module provides processing for the GRUB environment block.
Parsers included are:


GrubEnv - command ``grub2-editenv list``
----------------------------------------
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.grubenv)
class GrubEnv(Parser, dict):
    """
    This parser parses the output of the command ``grub2-editenv list``, which
    list the current variables in GRUB environment block file or error message.
    The module stores the key/value pairs in itself which inherits a dict.

    This parser used to handle the contents of the ``/boot/grub2/grubenv`` file
    before. Since the command output without error messages is the same as the
    content in ``/boot/grub2/grubenv``, and the ``grub2-editenv list`` command
    output will show the error messages if the file is corrupt or has serious
    errors in content, the current parser processes the ``grub2-editenv list``
    command and returns a dict of the grubenv variables.

    Cause the sosreports only collect the grubenv file and don't collect the
    ``grub2-editenv list`` command, the GrubEnv parser only handles the
    ``/boot/grub2/grubenv`` content for sos_archive.

    ``/boot/grub2/grubenv`` file is only used in Grub 2, but in RHEL8 with BLS
    being the default. There were several variables added that are referenced in
    the ``/boot/loader/entries/*.conf`` files.

    Sample output of the command::

        saved_entry=295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64
        kernelopts=root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200
        boot_success=0
        boot_indeterminate=2
        tuned_params=transparent_hugepages=never
        tuned_initrd=

    Examples:
        >>> type(grubenv)
        <class 'insights.parsers.grubenv.GrubEnv'>
        >>> grubenv.has_kernelopts
        True
        >>> grubenv.kernelopts
        'root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200'
        >>> grubenv.has_tuned_params
        True
        >>> grubenv.tuned_params
        'transparent_hugepages=never'
        >>> grubenv['saved_entry']
        '295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64'
    """

    def __init__(self, context):
        super(GrubEnv, self).__init__(context)

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty output.")

        self._errors = []
        data = dict()
        for line in get_active_lines(content):
            if "=" not in line:
                self._errors.append(line)
                continue

            key, value = line.split("=", 1)
            if not value:
                continue

            data[key] = value

        if (not data) and (not self._errors):
            raise SkipComponent("No parsed data.")

        self.update(data)

    @property
    def has_kernelopts(self):
        """ bool: Returns True/False depending on if kernelopts key is in the dict. """
        return "kernelopts" in self

    @property
    def kernelopts(self):
        """ str: Returns the string of kernelopts from the dict. """
        return self.get("kernelopts", "")

    @property
    def has_tuned_params(self):
        """ bool: Returns True/False depending of if the tuned_params key is in the dict. """
        return "tuned_params" in self

    @property
    def tuned_params(self):
        """ str: Returns the string of tuned_params from the dict."""
        return self.get("tuned_params", "")

    @property
    def errors(self):
        """ list: Returns the list of error message. """
        return self._errors



import pytest

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.specs import Specs
from insights.specs.datasources.ls import (
        list_with_la, list_with_la_filtered,
        list_with_lan, list_with_lan_filtered,
        list_with_lanL, list_with_lanR, list_with_lanRL,
        list_with_laRZ, list_with_laZ)


def setup_function(func):
    if func is test_la_empty:
        pass
    if func is test_la:
        filters.add_filter(Specs.ls_la_dirs, ["/"])
    if func is test_la_filterd:
        filters.add_filter(Specs.ls_la_filtered_dirs, ["/", '/boot'])
    if func is test_lan:
        filters.add_filter(Specs.ls_lan_dirs, ["/", '/boot'])
    if func is test_lan_filterd:
        filters.add_filter(Specs.ls_lan_filtered_dirs, ["/"])
    if func is test_lanL:
        filters.add_filter(Specs.ls_lanL_dirs, ["/", '/boot'])
    if func is test_lanR:
        filters.add_filter(Specs.ls_lanR_dirs, ["/"])
    if func is test_lanRL:
        filters.add_filter(Specs.ls_lanRL_dirs, ["/", '/boot'])
    if func is test_lanRZ:
        filters.add_filter(Specs.ls_laRZ_dirs, ['/boot'])
    if func is test_lanZ:
        filters.add_filter(Specs.ls_laZ_dirs, ["/", '/mnt'])


def teardown_function(func):
    for spec in (
            Specs.ls_la_dirs, Specs.ls_la_filtered, Specs.ls_la_filtered_dirs,
            Specs.ls_lan_dirs, Specs.ls_lan_filtered, Specs.ls_lan_filtered_dirs,
            Specs.ls_lanL_dirs, Specs.ls_lanR_dirs, Specs.ls_lanRL_dirs,
            Specs.ls_laRZ_dirs, Specs.ls_laZ_dirs):
        if spec in filters._CACHE:
            del filters._CACHE[spec]


def test_la_empty():
    with pytest.raises(SkipComponent):
        list_with_la({})


def test_la():
    ret = list_with_la({})
    assert ret == '/ _non_existing_'


def test_la_filterd():
    ret = list_with_la_filtered({})
    assert ret == '/ /boot'


def test_lan():
    ret = list_with_lan({})
    assert ret == '/ /boot'


def test_lan_filterd():
    ret = list_with_lan_filtered({})
    assert ret == '/ _non_existing_'


def test_lanL():
    ret = list_with_lanL({})
    assert ret == '/ /boot'


def test_lanR():
    ret = list_with_lanR({})
    assert '/ ' in ret


def test_lanRL():
    ret = list_with_lanRL({})
    assert ret == '/ /boot'


def test_lanRZ():
    ret = list_with_laRZ({})
    assert ret == '/boot'


def test_lanZ():
    ret = list_with_laZ({})
    assert ret == '/ /mnt'



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import readlink_e_mtab
from insights.tests import context_wrap

REAL_FILE_PATH = '''
/proc/4578/mounts
'''.strip()

BAD_FILE_PATH = ""


def test_doc_examples():
    env = {
        'mtab': readlink_e_mtab.ReadLinkEMtab(
            context_wrap(REAL_FILE_PATH)),
    }
    failed, total = doctest.testmod(readlink_e_mtab, globs=env)
    assert failed == 0


def test_readlink_e_mtab():
    mtab = readlink_e_mtab.ReadLinkEMtab(context_wrap(REAL_FILE_PATH))
    assert len(mtab.path) > 0
    assert mtab.path == REAL_FILE_PATH


def test_fail():
    with pytest.raises(SkipComponent) as e:
        readlink_e_mtab.ReadLinkEMtab(context_wrap(BAD_FILE_PATH))
    assert "No Data from command: readlink -e /etc/mtab" in str(e)



"""
Ansible Info
============
Provide information about the Ansible packages installed on a system.
"""
from insights.core.plugins import combiner
from insights.parsers.installed_rpms import InstalledRpms

ANSIBLE_TOWER_PKG = "ansible-tower"
ANSIBLE_AUTOMATION_HUB_PKG = "automation-hub"
ANSIBLE_CATALOG_WORKER_PKG = "catalog-worker"
ANSIBLE_AUTOMATION_CONTROLLER_PKG = "automation-controller"
ANSIBLE_PACKAGES = [
    ANSIBLE_TOWER_PKG,
    ANSIBLE_AUTOMATION_HUB_PKG,
    ANSIBLE_CATALOG_WORKER_PKG,
    ANSIBLE_AUTOMATION_CONTROLLER_PKG,
]


@combiner(InstalledRpms)
class AnsibleInfo(dict):
    """
    Provides information related to Ansible based on the RPMs installed.

    Provides properties to determine the Ansible specific system characteristics. The
    base class of the combiner is ``dict`` with dictionary keys being the Ansible
    package names, and data values being
    :py:class:`insights.parsers.installed_rpms.InstalledRpm` objects.
    See the :py:class:`insights.parsers.installed_rpms.InstalledRpm`
    class for more information on object methods and values.

    Properties are provided to aid in fingerprinting of the system.

    Examples:
        >>> type(info)
        <class 'insights.combiners.ansible_info.AnsibleInfo'>
        >>> info.is_tower
        True
        >>> info.tower_version
        '1.0.0'
        >>> info.is_controller
        True
        >>> info.controller_version
        '1.0.0'
    """
    def __init__(self, rpms):
        pkg_versions = dict([(pkg, rpms.get_max(pkg)) for pkg in ANSIBLE_PACKAGES if rpms.get_max(pkg) is not None])
        self.update(pkg_versions)

    @property
    def is_tower(self):
        """ bool: Whether or not this system has ``ansible-tower`` installed """
        return ANSIBLE_TOWER_PKG in self

    @property
    def tower_version(self):
        """ str: Version of ansible-tower installed or ``None``"""
        return self[ANSIBLE_TOWER_PKG].version if ANSIBLE_TOWER_PKG in self else None

    @property
    def is_controller(self):
        """
        bool: Whether or not this system has ``ansible-tower`` or
            ``automation-controller`` installed
        """
        return ANSIBLE_TOWER_PKG in self or ANSIBLE_AUTOMATION_CONTROLLER_PKG in self

    @property
    def controller_version(self):
        """
        str: Version of ansible-tower installed, or if it's not installed
            the version of automation-controller installed or ``None``
        """
        if ANSIBLE_TOWER_PKG in self:
            return self[ANSIBLE_TOWER_PKG].version
        elif ANSIBLE_AUTOMATION_CONTROLLER_PKG in self:
            return self[ANSIBLE_AUTOMATION_CONTROLLER_PKG].version

    @property
    def is_hub(self):
        """ bool: Whether or not this system has ``automation-hub`` installed """
        return ANSIBLE_AUTOMATION_HUB_PKG in self

    @property
    def hub_version(self):
        """ str: Version of automation-hub installed or ``None``"""
        return self[ANSIBLE_AUTOMATION_HUB_PKG].version if ANSIBLE_AUTOMATION_HUB_PKG in self else None

    @property
    def is_catalog_worker(self):
        """ bool: Whether or not this system has ``catalog-worker`` installed """
        return ANSIBLE_CATALOG_WORKER_PKG in self

    @property
    def catalog_worker_version(self):
        """ str: Version of catalog-worker installed or ``None``"""
        return self[ANSIBLE_CATALOG_WORKER_PKG].version if ANSIBLE_CATALOG_WORKER_PKG in self else None



"""
Datasources to collect the nginx configuration files from containers
"""
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import container_execute
from insights.specs import Specs
from insights.specs.datasources.container import running_rhel_containers


class LocalSpecs(Specs):
    """ Local specs used only by nginx container datasources """
    container_find_etc_opt_conf = container_execute(running_rhel_containers, "find /etc /opt -name '*.conf'")


@datasource(LocalSpecs.container_find_etc_opt_conf, HostContext)
def nginx_conf(broker):
    """
    Returns a list of tuple of (<podman|docker>, container_id, conf_path, image)
    """
    find_list = broker[LocalSpecs.container_find_etc_opt_conf]
    ret = []
    for conf_list in find_list:
        for conf_path in conf_list.content:
            # FIXME: refine the path filter
            if 'etc/nginx' in conf_path or 'rh-nginx' in conf_path:
                ret.append((conf_list.image, conf_list.engine, conf_list.container_id, conf_path))
    if ret:
        # Return list of tuple:
        # - (image, <podman|docker>, container_id, conf_path)
        return ret

    raise SkipComponent



"""
RabbitMQ Logs
=============

Module for parsing the log files for RabbitMQ:

RabbitMQLogs - file ``/var/log/rabbitmq/rabbit@$HOSTNAME.log``
--------------------------------------------------------------

RabbitMQStartupErrLog - file ``/var/log/rabbitmq/startup_err``
--------------------------------------------------------------

RabbitMQStartupLog - file ``/var/log/rabbitmq/startup_log``
-----------------------------------------------------------

"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.rabbitmq_startup_log)
class RabbitMQStartupLog(LogFileOutput):
    """Class for parsing ``/var/log/rabbitmq/startup_log`` file.

    Typical content of ``startup_log`` file is::

        Starting all nodes...
        Starting node rabbit@ubuntu...

        +---+   +---+
        |   |   |   |
        |   |   |   |
        |   |   |   |
        |   +---+   +-------+
        |                   |
        | RabbitMQ  +---+   |
        |           |   |   |
        |   v1.8.0  +---+   |
        |                   |
        +-------------------+
        AMQP 8-0
        Copyright (C) 2007-2010 LShift Ltd., Cohesive Financial Technologies LLC., and Rabbit Technologies Ltd.
        Licensed under the MPL.  See http://www.rabbitmq.com/

        node           : rabbit@ubuntu
        app descriptor : /usr/lib/rabbitmq/lib/rabbitmq_server-1.8.0/sbin/../ebin/rabbit.app
        home dir       : /var/lib/rabbitmq
        cookie hash    : mfoMkOc9CYok/SmH7RH9Jg==
        log            : /var/log/rabbitmq/rabbit@ubuntu.log
        sasl log       : /var/log/rabbitmq/rabbit@ubuntu-sasl.log
        database dir   : /var/lib/rabbitmq/mnesia/rabbit@ubuntu
        erlang version : 5.7.4

        starting file handle cache server                                     ...done
        starting worker pool                                                  ...done
        starting database                                                     ...done
        starting empty DB check                                               ...done
        starting exchange recovery                                            ...done
        starting queue supervisor and queue recovery                          ...BOOT ERROR: FAILED


    Note:
        Please refer to its super-class ``LogFileOutput``
    """
    pass


@parser(Specs.rabbitmq_startup_err)
class RabbitMQStartupErrLog(LogFileOutput):
    """Class for parsing ``/var/log/rabbitmq/startup_err`` file.

    Typical content of ``startup_err`` file is::

        Error: {node_start_failed,normal}

        Crash dump was written to: erl_crash.dump
        Kernel pid terminated (application_controller) ({application_start_failure,kernel,{shutdown,{kernel,start,[normal,[]]}}})

    .. note::
        Please refer to its super-class :py:class:`insights.core.LogFileOutput`
    """
    pass


@parser(Specs.rabbitmq_logs)
class RabbitMQLogs(LogFileOutput):
    """Class for parsing ``/var/log/rabbitmq/rabbit@$HOSTNAME.log`` file

    Typical content of ``rabbit@$HOSTNAME.log`` file is::

        =INFO REPORT==== 9-Nov-2016::14:29:11 ===
        Starting RabbitMQ 3.6.3 on Erlang 18.3.4.4
        Copyright (C) 2007-2016 Pivotal Software, Inc.
        Licensed under the MPL.  See http://www.rabbitmq.com/

        =INFO REPORT==== 9-Nov-2016::14:29:11 ===
        node           : rabbit@overcloud-controller-0
        home dir       : /var/lib/rabbitmq
        config file(s) : /etc/rabbitmq/rabbitmq.config
        cookie hash    : F7g8XhNTzvEK3KywLHh9yA==
        log            : /var/log/rabbitmq/rabbit@overcloud-controller-0.log
        sasl log       : /var/log/rabbitmq/rabbit@overcloud-controller-0-sasl.log
        database dir   : /var/lib/rabbitmq/mnesia/rabbit@overcloud-controller-0
        ...

    .. note::
        Please refer to its super-class :py:class:`insights.core.LogFileOutput`
        for full usage.

    .. note::
        Because this parser is defined using a PatternSpec, which returns
        multiple files, the data in the shared parser state is a list of
        these parser objects.  This means that for the moment you will have
        to iterate across these objects directly.

    Examples:
        >>> for log in shared[RabbitMQLogs]:
        ...     print 'log file:', log.file_path
        ...     print 'INFO lines:', len(log.get('INFO REPORT'))
        ...     print 'ERROR lines:', len(log.get('ERROR REPORT'))
        ...
        log file: /var/log/rabbitmq/rabbit@queue.example.com.log
        INFO lines: 2
        ERROR lines: 0
    """
    time_format = '%d-%b-%Y::%H:%M:%S'



"""
Iris - InterSystems
===================
Parsers included in this module are:

IrisList - Command ``/usr/bin/iris list``
=========================================

IrisCpf - Files ``iris.cpf``
============================

IrisMessages - Files ``messages.log``
=====================================
"""

from insights.core import CommandParser, IniConfigFile, LogFileOutput
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.iris_list)
class IrisList(CommandParser, list):
    """
    Parse the output of the ``/usr/bin/iris list`` command.

    Attributes:
        default (dict): the default instance

    Sample Input::

        Configuration 'IRIS'   (default)
            directory:    /intersystems
            versionid:    2023.1.0.235.1com
            datadir:      /intersystems
            conf file:    iris.cpf  (SuperServer port = 1972, WebServer = 52773)
            status:       running, since Tue Jun 27 01:55:25 2023
            state:        ok
            product:      InterSystems IRIS

    Examples:
        >>> iris_info[0]['status']
        'running, since Tue Jun 27 01:55:25 2023'
        >>> iris_info.is_running
        True
        >>> iris_info.default['status']
        'running, since Tue Jun 27 01:55:25 2023'
    """

    def parse_content(self, content):
        self.default = {}
        item_instance = {}

        for line in content:
            if not line.strip():
                continue
            if line.strip().startswith('Configuration'):
                instance_name = line.split()[1].strip('\'"')
                item_instance = {"instance_name": instance_name}
                if "(default)" in line:
                    self.default = item_instance
                self.append(item_instance)
            elif ":" in line and item_instance:
                key, value = line.split(":", 1)
                item_instance[key.strip()] = value.strip()

        if len(self) == 0:
            raise SkipComponent("The result is empty")

    @property
    def is_running(self):
        """Return True when the iris instance is running, and False when it is down"""
        return any(item.get('status', "").startswith('running') for item in self)


@parser(Specs.iris_cpf)
class IrisCpf(IniConfigFile):
    """
    Parse the content of "iris.cpf" file.

    Sample Input::

        [ConfigFile]
        Product=IRIS
        Version=2023.1

        [Databases]
        IRISSYS=/intersystems/mgr/
        IRISLIB=/intersystems/mgr/irislib/
        IRISTEMP=/intersystems/mgr/iristemp/
        IRISLOCALDATA=/intersystems/mgr/irislocaldata/
        IRISAUDIT=/intersystems/mgr/irisaudit/
        ENSLIB=/intersystems/mgr/enslib/
        USER=/intersystems/mgr/user/

        [Namespaces]
        %SYS=IRISSYS
        USER=USER

    Examples:
        >>> "Namespaces" in iris_cpf
        True
        >>> iris_cpf.has_option('Databases', 'IRISSYS')
        True
        >>> iris_cpf.get("Databases", "IRISSYS")
        '/intersystems/mgr/'
    """
    pass


@parser(Specs.iris_messages_log)
class IrisMessages(LogFileOutput):
    """
    Parse the content of "messages.log" file.

    Sample Input::

        06/26/23-08:02:17:828 (144145) 0 [Generic.Event] Allocated 495MB shared memory
        06/26/23-08:02:17:828 (144145) 0 [Generic.Event] 32MB global buffers, 80MB routine buffers, 64MB journal buffers, 4MB buffer descriptors, 300MB heap, 5MB ECP, 9MB miscellaneous
        06/26/23-08:02:17:831 (144145) 0 [Crypto.IntelSandyBridgeAESNI] Intel Sandy Bridge AES-NI instructions detected.
        06/26/23-08:02:17:831 (144145) 0 [SIMD] SIMD optimization level: DEFAULT X86
        06/26/23-08:02:17:903 (144145) 0 [WriteDaemon.UsingWIJFile] Using WIJ file: /intersystems/mgr/IRIS.WIJ
        06/26/23-08:02:17:903 (144145) 0 [Generic.Event] No journaling info from prior system
        06/26/23-08:02:18:110 (144145) 0 [Generic.Event]
        Startup of InterSystems IRIS [IRIS for UNIX (Red Hat Enterprise Linux 8 for x86-64) 2023.1 (Build 235.1) Fri Jun 2 2023 13:26:47 EDT]
                in ../bin/
                with mgr: /intersystems/mgr
                with wij: /intersystems/mgr/IRIS.WIJ
                from: /intersystems/mgr/
        OS=[Linux], version=[#1 SMP Fri Sep 30 11:45:06 EDT 2022], release=[4.18.0-425.3.1.el8.x86_64], machine=[x86_64]
        nodename=[rhel8].
        numasyncwijbuf: 2, wdwrite_asyncio_max: 64, wijdirectio: on, synctype: 3
        System Initialized.
        06/26/23-08:02:18:136 (144182) 0 [WriteDaemon.Started] Write daemon started.
                [Namespaces]
                %SYS=IRISSYS
                USER=USER

    Examples:
        >>> len(list(iris_log.get('Generic.Event')))
        4
    """
    time_format = '%m/%d/%y-%H:%M:%S:%f'



"""
Cloud Instance
==============

Combiner for the basic information of a cloud instance. It combines the
results of the following combiners and parsers:

* :py:class:`insights.combiners.cloud_provider.CloudProvider`
* :py:class:`insights.parsers.aws_instance_id.AWSInstanceIdDoc`
* :py:class:`insights.parsers.azure_instance.AzureInstanceID`
* :py:class:`insights.parsers.azure_instance.AzureInstanceType`
* :py:class:`insights.parsers.gcp_instance_type.GCPInstanceType`
* :py:class:`insights.parsers.subscription_manager.SubscriptionManagerFacts`

"""
from insights.combiners.cloud_provider import CloudProvider
from insights.core.exceptions import ContentException, SkipComponent
from insights.core.filters import add_filter
from insights.core.plugins import combiner
from insights.parsers.aws_instance_id import AWSInstanceIdDoc
from insights.parsers.azure_instance import AzureInstanceID, AzureInstanceType
from insights.parsers.gcp_instance_type import GCPInstanceType
from insights.parsers.subscription_manager import SubscriptionManagerFacts

add_filter(SubscriptionManagerFacts, 'instance_id')


@combiner(
    CloudProvider,
    [
        AWSInstanceIdDoc,
        AzureInstanceID,
        AzureInstanceType,
        GCPInstanceType,
        SubscriptionManagerFacts,
    ]
)
class CloudInstance(object):
    """
    Class to provide the basic information of a cloud instance.

    Attributes:
        provider (str): The cloud provider, e.g. "aws", "azure", "ibm",
                "google", or "alibaba". It's from the value of
                :class:`insights.combiners.cloud_provider.CloudProvider.cloud_provider`
        id (str): The ID of the cloud instance
        type (str): The type of the cloud instance.
                Different cloud providers have different illustration of the
                `type` and `size`, here for this Combiner, we treat the `type` and
                `size` as the same.  E.g.::

                    - "Standard_L64s_v2" for Azure
                    - "x1.16xlarge" for AWS
                    - "m1-megamem-96" for GCP

        size (str): Alias of the `type`

    Examples:
        >>> ci.provider
        'aws'
        >>> ci.id == 'i-1234567890abcdef0'
        True
        >>> ci.type == 't2.micro'
        True
        >>> ci.size == 't2.micro'
        True
    """
    def __init__(self, cp, aws=None, azure_id=None, azure_type=None,
                 gcp=None, facts=None):
        self.provider = cp.cloud_provider
        self.id = None
        # 1. Get from the Cloud REST API at first
        if aws:
            self.id = aws.get('instanceId')
            self.type = aws.get('instanceType')
        elif azure_id and azure_type:
            self.id = azure_id.id
            self.type = azure_type.raw
        elif gcp:
            self.type = gcp.raw
        # 2. Check the "subscription-manager facts"
        if self.id is None and facts:
            key = "{0}_instance_id".format(self.provider)
            if key not in facts:
                raise ContentException("Unmatched/unsupported types!")
            self.id = facts[key]
        # The instance id is the key attribute of this Combiner
        if self.id is None:
            raise SkipComponent
        # 'size' is the alias of 'type'
        self.size = self.type



"""
SosConf - file ``/etc/sos.conf`` or ``/etc/sos/sos.conf``
=========================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.sos_conf)
class SosConf(IniConfigFile):
    """
    This class provides parsing for the file ``/etc/sos.conf``.
    On RHEL 8 and later, the file location is ``/etc/sos/sos.conf``.

    Sample input data is in the format::

        [general]
        #verbose = 3
        #verify = yes
        batch = yes
        #log-size = 15

        [plugins]
        disable = rpm, selinux, dovecot

        [tunables]
        #rpm.rpmva = off

    Examples:
        >>> type(sos_conf)
        <class 'insights.parsers.sos_conf.SosConf'>
        >>> 'plugins' in sos_conf.sections()
        True
        >>> sos_conf.get("plugins", "disable")
        'rpm, selinux, dovecot'
        >>> sos_conf.getboolean("general", "batch")
        True
    """
    pass



from insights.tests import context_wrap
from insights.parsers import jboss_domain_log
from insights.parsers.jboss_domain_log import JbossDomainServerLog, JbossStandaloneServerLog
from datetime import time
import doctest

OUT1 = """
16:22:57,476 INFO  [org.xnio] (MSC service thread 1-12) XNIO Version 3.0.14.GA-redhat-1
16:22:57,480 INFO  [org.xnio.nio] (MSC service thread 1-12) XNIO NIO Implementation Version 3.0.14.GA-redhat-1
16:22:57,495 INFO  [org.jboss.remoting] (MSC service thread 1-12) JBoss Remoting version 3.3.5.Final-redhat-1
16:23:03,881 INFO  [org.jboss.as.controller.management-deprecated] (ServerService Thread Pool -- 23) JBAS014627: Attribute 'enabled' in the resource at address '/subsystem=datasources/data-source=ExampleDS' is deprecated, and may be removed in future version. See the attribute description in the output of the read-resource-description operation to learn more about the deprecation.
16:23:03,958 INFO  [org.jboss.as.security] (ServerService Thread Pool -- 37) JBAS013371: Activating Security Subsystem
16:23:03,960 INFO  [org.jboss.as.webservices] (ServerService Thread Pool -- 33) JBAS015537: Activating WebServices Extension
16:23:03,957 WARN  [org.jboss.as.txn] (ServerService Thread Pool -- 35) JBAS010153: Node identifier property is set to the default value. Please make sure it is unique.
16:23:03,962 INFO  [org.jboss.as.security] (MSC service thread 1-11) JBAS013370: Current PicketBox version=4.1.1.Final-redhat-1
16:23:03,974 INFO  [org.jboss.as.naming] (ServerService Thread Pool -- 42) JBAS011800: Activating Naming Subsystem
16:23:04,024 INFO  [org.jboss.as.clustering.infinispan] (ServerService Thread Pool -- 54) JBAS010280: Activating Infinispan subsystem.
16:23:04,049 INFO  [org.jboss.as.naming] (MSC service thread 1-8) JBAS011802: Starting Naming Service
16:23:04,058 INFO  [org.jboss.as.jacorb] (ServerService Thread Pool -- 53) JBAS016300: Activating JacORB Subsystem
16:23:04,066 INFO  [org.jboss.as.configadmin] (ServerService Thread Pool -- 58) JBAS016200: Activating ConfigAdmin Subsystem
16:23:04,077 INFO  [org.jboss.as.connector.logging] (MSC service thread 1-6) JBAS010408: Starting JCA Subsystem (IronJacamar 1.0.32.Final-redhat-1)
16:23:04,079 INFO  [org.jboss.as.connector.subsystems.datasources] (ServerService Thread Pool -- 57) JBAS010403: Deploying JDBC-compliant driver class org.h2.Driver (version 1.3)
16:23:04,080 INFO  [org.jboss.as.jsf] (ServerService Thread Pool -- 46) JBAS012615: Activated the following JSF Implementations: [main, 1.2]
16:23:04,267 INFO  [org.jboss.as.mail.extension] (MSC service thread 1-14) JBAS015400: Bound mail session [java:jboss/mail/Default]
16:23:04,366 INFO  [org.jboss.jaxr] (MSC service thread 1-16) JBAS014000: Started JAXR subsystem, binding JAXR connection factory into JNDI as: java:jboss/jaxr/ConnectionFactory
16:23:04,726 INFO  [org.apache.coyote.http11.Http11Protocol] (MSC service thread 1-4) JBWEB003001: Coyote HTTP/1.1 initializing on : http-/192.168.199.175:8080
16:23:04,745 INFO  [org.apache.coyote.http11.Http11Protocol] (MSC service thread 1-4) JBWEB003000: Coyote HTTP/1.1 starting on: http-/192.168.199.175:8080
16:23:04,801 INFO  [org.hornetq.core.server] (ServerService Thread Pool -- 60) HQ221000: live server is starting with configuration HornetQ Configuration (clustered=false,backup=false,sharedStore=true,journalDirectory=/home/test/jboss/machine2/domain/servers/server-one/data/messagingjournal,bindingsDirectory=/home/test/jboss/machine2/domain/servers/server-one/data/messagingbindings,largeMessagesDirectory=/home/test/jboss/machine2/domain/servers/server-one/data/messaginglargemessages,pagingDirectory=/home/test/jboss/machine2/domain/servers/server-one/data/messagingpaging)
16:23:04,809 INFO  [org.hornetq.core.server] (ServerService Thread Pool -- 60) HQ221006: Waiting to obtain live lock
16:23:04,969 INFO  [org.hornetq.core.server] (ServerService Thread Pool -- 60) HQ221013: Using NIO Journal
ectory=/home/test/jboss/machine2/domain/servers/server-one/data/messagingjournal,bindingsDirectory=/home/test/jboss/machine2/domain/servers/server-one/data/messagingbindings,largeMessagesDirectory=/home/test/jboss/machine2/domain/servers/server-one/data/messaginglargemessages,pagingDirectory=/home/test/jboss/machine2/domain/servers/server-one/data/messagingpaging)
16:23:05,018 INFO  [org.jboss.as.remoting] (MSC service thread 1-7) JBAS017100: Listening on 192.168.199.175:444
""".strip()


def test_jboss_domain_server_log():
    out_log = JbossDomainServerLog(
        context_wrap(OUT1,
                     path="/home/test/jboss/machine2/domain/servers/server-one/log/server.log"))
    assert out_log.file_path == "/home/test/jboss/machine2/domain/servers/server-one/log/server.log"
    assert "XNIO Version 3.0.14.GA-redhat-1" in out_log
    assert len(out_log.get("GA-redhat-1")) == 2
    assert out_log.get("GA-redhat-1")[0].get(
        "raw_message") == "16:22:57,476 INFO  [org.xnio] (MSC service thread 1-12) XNIO Version 3.0.14.GA-redhat-1"
    assert "Listening on 192.168.199.175:444" in out_log
    assert len(list(out_log.get_after(time(16, 23, 0o4)))) == 16


OUT2 = """
16:22:57,476 INFO  [org.xnio] (MSC service thread 1-12) XNIO Version 3.0.14.GA-redhat-1
16:22:57,480 INFO  [org.xnio.nio] (MSC service thread 1-12) XNIO NIO Implementation Version 3.0.14.GA-redhat-1
16:22:57,495 INFO  [org.jboss.remoting] (MSC service thread 1-12) JBoss Remoting version 3.3.5.Final-redhat-1
16:23:03,881 INFO  [org.jboss.as.controller.management-deprecated] (ServerService Thread Pool -- 23) JBAS014627: Attribute 'enabled' in the resource at address '/subsystem=datasources/data-source=ExampleDS' is deprecated, and may be removed in future version. See the attribute description in the output of the read-resource-description operation to learn more about the deprecation.
16:23:03,958 INFO  [org.jboss.as.security] (ServerService Thread Pool -- 37) JBAS013371: Activating Security Subsystem
""".strip()

standalong_server_log = """
2018-07-17 10:58:44,606 INFO  [org.jboss.modules] (main) JBoss Modules version 1.6.0.Final-redhat-1
2018-07-17 10:58:44,911 INFO  [org.jboss.msc] (main) JBoss MSC version 1.2.7.SP1-redhat-1
2018-07-17 10:58:45,032 INFO  [org.jboss.as] (MSC service thread 1-7) WFLYSRV0049: JBoss EAP 7.1.0.GA (WildFly Core 3.0.10.Final-redhat-1) starting
2018-07-17 10:58:45,033 DEBUG [org.jboss.as.config] (MSC service thread 1-7) Configured system properties:
    [Standalone] =
    awt.toolkit = sun.awt.X11.XToolkit
    file.encoding = UTF-8
    file.encoding.pkg = sun.io
    file.separator = /
    java.awt.graphicsenv = sun.awt.X11GraphicsEnvironment
    java.awt.headless = true
    java.awt.printerjob = sun.print.PSPrinterJob
    java.class.path = /opt/jboss-eap-7.1/jboss-modules.jar
    java.class.version = 52.0
    java.endorsed.dirs = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/endorsed
    java.ext.dirs = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/ext:/usr/java/packages/lib/ext
    java.home = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre
    java.io.tmpdir = /tmp
    java.library.path = /usr/java/packages/lib/amd64:/usr/lib64:/lib64:/lib:/usr/lib
    java.net.preferIPv4Stack = true
    java.runtime.name = OpenJDK Runtime Environment
    java.runtime.version = 1.8.0_111-b16
    java.specification.name = Java Platform API Specification
    java.specification.vendor = Oracle Corporation
    java.specification.version = 1.8
    java.util.logging.manager = org.jboss.logmanager.LogManager
    java.vendor = Oracle Corporation
    java.vendor.url = http://java.oracle.com/
    java.vendor.url.bug = http://bugreport.sun.com/bugreport/
    java.version = 1.8.0_111
    java.vm.info = mixed mode
    java.vm.name = OpenJDK 64-Bit Server VM
    java.vm.specification.name = Java Virtual Machine Specification
    java.vm.specification.vendor = Oracle Corporation
    java.vm.specification.version = 1.8
    java.vm.vendor = Oracle Corporation
    java.vm.version = 25.111-b16
    javax.management.builder.initial = org.jboss.as.jmx.PluggableMBeanServerBuilder
    javax.xml.datatype.DatatypeFactory = __redirected.__DatatypeFactory
    javax.xml.parsers.DocumentBuilderFactory = __redirected.__DocumentBuilderFactory
    javax.xml.parsers.SAXParserFactory = __redirected.__SAXParserFactory
    javax.xml.stream.XMLEventFactory = __redirected.__XMLEventFactory
    javax.xml.stream.XMLInputFactory = __redirected.__XMLInputFactory
    javax.xml.stream.XMLOutputFactory = __redirected.__XMLOutputFactory
    javax.xml.transform.TransformerFactory = __redirected.__TransformerFactory
    javax.xml.validation.SchemaFactory:http://www.w3.org/2001/XMLSchema = __redirected.__SchemaFactory
    javax.xml.xpath.XPathFactory:http://java.sun.com/jaxp/xpath/dom = __redirected.__XPathFactory
    jboss.home.dir = /opt/jboss-eap-7.1
    jboss.host.name = mylinux
    jboss.modules.dir = /opt/jboss-eap-7.1/modules
    jboss.modules.system.pkgs = org.jboss.byteman
    jboss.node.name = mylinux
    jboss.qualified.host.name = mylinux
    jboss.server.base.dir = /opt/jboss-eap-7.1/standalone
    jboss.server.config.dir = /opt/jboss-eap-7.1/standalone/configuration
    jboss.server.data.dir = /opt/jboss-eap-7.1/standalone/data
    jboss.server.deploy.dir = /opt/jboss-eap-7.1/standalone/data/content
    jboss.server.log.dir = /opt/jboss-eap-7.1/standalone/log
    jboss.server.name = mylinux
    jboss.server.persist.config = true
    jboss.server.temp.dir = /opt/jboss-eap-7.1/standalone/tmp
    line.separator =

    logging.configuration = file:/opt/jboss-eap-7.1/standalone/configuration/logging.properties
    module.path = /opt/jboss-eap-7.1/modules
    org.jboss.boot.log.file = /opt/jboss-eap-7.1/standalone/log/server.log
    org.jboss.resolver.warning = true
    org.xml.sax.driver = __redirected.__XMLReaderFactory
    os.arch = amd64
    os.name = Linux
    os.version = 4.8.13-100.fc23.x86_64
    path.separator = :
    sun.arch.data.model = 64
    sun.boot.class.path = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/resources.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/rt.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/sunrsasign.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/jsse.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/jce.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/charsets.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/jfr.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/classes
    sun.boot.library.path = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/amd64
    sun.cpu.endian = little
    sun.cpu.isalist =
    sun.desktop = gnome
    sun.io.unicode.encoding = UnicodeLittle
    sun.java.command = /opt/jboss-eap-7.1/jboss-modules.jar -mp /opt/jboss-eap-7.1/modules org.jboss.as.standalone -Djboss.home.dir=/opt/jboss-eap-7.1 -Djboss.server.base.dir=/opt/jboss-eap-7.1/standalone --server-config=standalone-ha.xml
    sun.java.launcher = SUN_STANDARD
    sun.jnu.encoding = UTF-8
    sun.management.compiler = HotSpot 64-Bit Tiered Compilers
    sun.os.patch.level = unknown
    user.country = US
    user.dir = /opt/jboss-eap-7.1/bin
    user.home = /home/lizhong
    user.language = en
    user.name = lizhong
    user.timezone = Asia/Shanghai
2018-07-17 10:58:45,033 DEBUG [org.jboss.as.config] (MSC service thread 1-7) VM Arguments: -D[Standalone] -verbose:gc -Xloggc:/opt/jboss-eap-7.1/standalone/log/gc.log -XX:+PrintGCDetails -XX:+PrintGCDateStamps -XX:+UseGCLogFileRotation -XX:NumberOfGCLogFiles=5 -XX:GCLogFileSize=3M -XX:-TraceClassUnloading -Xms1303m -Xmx1303m -XX:MetaspaceSize=96M -XX:MaxMetaspaceSize=256m -Djava.net.preferIPv4Stack=true -Djboss.modules.system.pkgs=org.jboss.byteman -Djava.awt.headless=true -Dorg.jboss.boot.log.file=/opt/jboss-eap-7.1/standalone/log/server.log -Dlogging.configuration=file:/opt/jboss-eap-7.1/standalone/configuration/logging.properties
2018-07-17 10:58:46,021 INFO  [org.jboss.as.controller.management-deprecated] (Controller Boot Thread) WFLYCTL0028: Attribute 'security-realm' in the resource at address '/core-service=management/management-interface=http-interface' is deprecated, and may be removed in future version. See the attribute description in the output of the read-resource-description operation to learn more about the deprecation.
2018-07-17 10:58:46,033 INFO  [org.wildfly.security] (ServerService Thread Pool -- 30) ELY00001: WildFly Elytron version 1.1.7.Final-redhat-1
2018-07-17 10:58:46,036 INFO  [org.jboss.as.controller.management-deprecated] (ServerService Thread Pool -- 15) WFLYCTL0028: Attribute 'security-realm' in the resource at address '/subsystem=undertow/server=default-server/https-listener=https' is deprecated, and may be removed in future version. See the attribute description in the output of the read-resource-description operation to learn more about the deprecation.
2018-07-17 10:58:46,172 INFO  [org.jboss.as.server] (Controller Boot Thread) WFLYSRV0039: Creating http management service using socket-binding (management-http)
"""


def test_jboss_standalone_server_log():
    out_log = JbossStandaloneServerLog(
        context_wrap(standalong_server_log, path="JBOSS_HOME/standalone/log/server.log"))
    assert out_log.file_path == "/JBOSS_HOME/standalone/log/server.log"
    assert "sun.java.command =" in out_log
    assert len(out_log.get("sun.java.command =")) == 1
    assert out_log.get("java.specification.")[-1][
               "raw_message"].strip() == "java.specification.version = 1.8"


def test_jboss_domain_server_log_doc_examples():
    env = {
        'JbossDomainServerLog': JbossDomainServerLog,
        'log': JbossDomainServerLog(context_wrap(OUT2,
                                                 path='/home/test/jboss/machine2/domain/servers/server-one/log/server.log')),
        'JbossStandaloneServerLog': JbossStandaloneServerLog,
        'standalone_log': JbossStandaloneServerLog(context_wrap(standalong_server_log,
                                                                path="/JBOSS_HOME/standalone/log/server.log")),
    }
    failed, total = doctest.testmod(jboss_domain_log, globs=env)
    assert failed == 0



from insights.tests import context_wrap
from insights.parsers.sysconfig import VirtWhoSysconfig

VIRTWHO = """
# Register ESX machines using vCenter
#VIRTWHO_ESX=0
# Register guests using RHEV-M
 VIRTWHO_RHEVM=1

# Options for RHEV-M mode
VIRTWHO_RHEVM_OWNER=

TEST_OPT="A TEST"
""".strip()


def test_sysconfig_virt_who():
    result = VirtWhoSysconfig(context_wrap(VIRTWHO))
    assert result["VIRTWHO_RHEVM"] == '1'
    assert result["VIRTWHO_RHEVM_OWNER"] == ''
    assert result.get("NO_SUCH_OPTIONS") is None
    assert "NOSUCHOPTIONS" not in result
    assert result.get("TEST_OPT") == 'A TEST'



"""
SystemctlStatusAll command ``systemctl status --all``
=====================================================
"""

from insights.core import LogFileOutput
from insights.core.plugins import parser
from insights.core.filters import add_filter
from insights.specs import Specs


add_filter(Specs.systemctl_status_all, ['State: ', 'Jobs: ', 'Failed: '])


@parser(Specs.systemctl_status_all)
class SystemctlStatusAll(LogFileOutput):
    """
    Class for parsing the output from command ``systemctl status --all``.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`

    Sample output::

        * redhat.test.com
            State: degraded
             Jobs: 0 queued
           Failed: 2 units
            Since: Thu 2021-09-23 12:03:43 UTC; 3h 7min ago
           CGroup: /
                   |-user.slice
                   | `-user-1000.slice
                   |   |-user@1000.service

        Sep 23 15:11:07 redhat.test.com systemd[1]: proc-sys-fs-binfmt_misc.automount: Automount point already active?
        Sep 23 15:11:07 redhat.test.com systemd[1]: proc-sys-fs-binfmt_misc.automount: Got automount request for /proc/sys/fs/binfmt_mis

    Examples:
        >>> type(systemctl_status)
        <class 'insights.parsers.systemctl_status_all.SystemctlStatusAll'>
        >>> "Automount point already active?" in  systemctl_status
        True
        >>> systemctl_status.state == 'degraded'
        True
        >>> systemctl_status.jobs == '0 queued'
        True
        >>> systemctl_status.failed == '2 units'
        True
    """
    _state = _jobs = _failed = False  # Use False to avoid conflicting with None

    def __get_item(self, key):
        ret = self.get(key, num=1)
        if ret:
            return ret[0]['raw_message'].split(':')[1].strip()
        return None

    @property
    def state(self):
        """
        (str): "State" shown in the command output, `None` by default.
        """
        if self._state is False:
            self._state = self.__get_item('State: ')
        return self._state

    @property
    def jobs(self):
        """
        (str): "Jobs" shown in the command output, `None` by default.
        """
        if self._jobs is False:
            self._jobs = self.__get_item('Jobs: ')
        return self._jobs

    @property
    def failed(self):
        """
        (str): "Failed" shown in the command output, `None` by default.
        """
        if self._failed is False:
            self._failed = self.__get_item('Failed: ')
        return self._failed



"""
Repquota - command ``repquota``
===============================

Parser contains in this module is:

RepquotaAGNPUV - command ``repquota -agnpuv``
---------------------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.repquota_agnpuv)
class RepquotaAGNPUV(CommandParser):
    """
    `repquota -agnpuv` prints a summary of the disc usage and quotas information for the specified file systems.

    Typical content looks like::

        *** Report for user quotas on device /dev/sdb
        Block grace time: 7days; Inode grace time: 7days
                                Block limits                File limits
        User            used    soft    hard  grace    used  soft  hard  grace
        ----------------------------------------------------------------------
        #0        --    5120       0       0      0       4     0     0      0
        #1009     +-   61440   51200  102400 1679983770       1     0     0      0
        #1010     --       0   51200  102400      0       0     0     0      0
        #1011     +-  107520   51200  307200 1680640819       2     0     0      0
        #1012     --   51200       0       0      0       1     0     0      0
        #1013     --       0   51200  102400      0       0     0     0      0

        *** Status for user quotas on device /dev/sdb
        Accounting: ON; Enforcement: ON
        Inode: #131 (2 blocks, 2 extents)

        *** Report for group quotas on device /dev/sdb
        Block grace time: 7days; Inode grace time: 7days
                                Block limits                File limits
        Group           used    soft    hard  grace    used  soft  hard  grace
        ----------------------------------------------------------------------
        #0        --    5120       0       0      0       4     0     0      0
        #1004     --   61440       0       0      0       1     0     0      0
        #1005     --   51200  972800 1048576      0       1     0     0      0
        #1011     --  107520       0       0      0       2     0     0      0

        *** Status for group quotas on device /dev/sdb
        Accounting: ON; Enforcement: ON
        Inode: #132 (2 blocks, 2 extents)

    Examples:
        >>> type(repquota)
        <class 'insights.parsers.repquota.RepquotaAGNPUV'>
        >>> 'enforcement' in repquota.user_quota['/dev/sdb']
        True
        >>> repquota.group_quota['/dev/sdb']['quota_info'][0] == {'group': '0', 'status': '--', 'block_used': '5120', 'block_soft': '0', 'block_hard': '0', 'block_grace': '0', 'file_used': '4', 'file_soft': '0', 'file_hard': '0', 'file_grace': '0'}
        True

    Raises:
        insights.core.exceptions.SkipComponent: if the output of the ``repquota -agnpuv`` command is empty.
    """

    def parse_content(self, content):
        self.user_quota = {}
        self.group_quota = {}
        data = device = heading = quota_list = None
        for line in content:
            if not line.strip():
                continue

            if line.startswith(('*** Report for', '*** Status for')):
                device = line.split()[-1]
                heading = ['group', 'block_used', 'block_soft', 'block_hard', 'block_grace', 'file_used', 'file_soft', 'file_hard', 'file_grace']
                data = self.group_quota
                if ' user ' in line:
                    heading[0] = 'user'
                    data = self.user_quota
                data[device] = data.get(device, dict(quota_info=[]))
                quota_list = data.get(device).get('quota_info')

            elif device and line.startswith('#'):
                line_sp = line.strip('#').split()
                info = dict(status=line_sp.pop(1))
                info.update(zip(heading, line_sp))
                quota_list.append(info)

            elif device and 'Accounting' in line and 'Enforcement' in line:
                data[device].update(enforcement='Enforcement: ON' in line,
                                    accounting='Accounting: ON' in line)

        if not self.user_quota and not self.group_quota:
            raise SkipComponent("Empty result")



from insights.parsr import Char, Opt


def test_opt():
    a = Opt(Char("a"))
    assert a("a") == "a"
    assert a("b") is None


def test_opt_default():
    a = Opt(Char("a"), "Default")
    assert a("b") == "Default"



"""
KernelConf - file ``/boot/config-*``
====================================

This parser parses the content from kernel config file for
individual installed kernel. This parser will return the data
in dictionary format.

Sample Content from ``/boot/config-3.10.0-862.el7.x86_64``::


    #
    # Automatically generated file; DO NOT EDIT.
    # Linux/x86_64 3.10.0-693.el7.x86_64 Kernel Configuration
    #
    CONFIG_64BIT=y
    CONFIG_X86_64=y
    CONFIG_X86=y
    CONFIG_INSTRUCTION_DECODER=y
    CONFIG_OUTPUT_FORMAT="elf64-x86-64"
    CONFIG_ARCH_DEFCONFIG="arch/x86/configs/x86_64_defconfig"
    CONFIG_ARCH_MMAP_RND_COMPAT_BITS_MIN=8
    CONFIG_PREEMPT_RT_FULL=y


Examples:
    >>> type(kconfig)
    <class 'insights.parsers.kernel_config.KernelConf'>
    >>> kconfig.get("CONFIG_PREEMPT_RT_FULL") == "y"
    True
    >>> len(kconfig) == 8
    True
    >>> kconfig.kconf_file
    'config-3.10.0-327.28.3.rt56.235.el7.x86_64'
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import split_kv_pairs
from insights.specs import Specs


@parser(Specs.kernel_config)
class KernelConf(Parser, dict):

    """
    Parase `/boot/config-*` file, returns a dict contains kernel
    configurations.

    """

    def parse_content(self, content):
        if (not content) or (not self.file_path):
            raise SkipComponent("No Contents")

        self._config_name = self.file_path.rsplit("/")[-1]
        lines = [l for l in content if not l.strip().startswith('#')]
        self.update(split_kv_pairs(lines, ordered=True))

    @property
    def kconf_file(self):
        """
        (str): It will return the kernel config file.
        """
        return self._config_name



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import mount
from insights.parsers.mount import Mount, ProcMounts, MountInfo
from insights.tests import context_wrap

MOUNT_DATA = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
nfsd on /proc/fs/nfsd type nfsd (rw,relatime)
/dev/sda1 on /boot type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/fedora-home on /home type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/fedora-root on / type ext4 (rw,relatime,seclabel,data=ordered)
sunrpc on /var/lib/nfs/rpc_pipefs type rpc_pipefs (rw,relatime)
tmpfs on /run/user/42 type tmpfs (rw,nosuid,nodev,relatime,seclabel,size=1605428k,mode=700,uid=42,gid=42)
tmpfs on /run/user/1000 type tmpfs (rw,nosuid,nodev,relatime,seclabel,size=1605428k,mode=700,uid=1000,gid=1000)
gvfsd-fuse on /run/user/1000/gvfs type fuse.gvfsd-fuse (rw,nosuid,nodev,relatime,user_id=1000,group_id=1000)
fusectl on /sys/fs/fuse/connections type fusectl (rw,relatime)
/dev/mapper/HostVG-Config on /etc/shadow type ext4 (rw,noatime,seclabel,stripe=256,data=ordered) [CONFIG]
/dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]
""".strip()

# Missing 'on' in second line
MOUNT_ERR_DATA = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
/dev/mapper/fedora-root on / type ext4 (rw,relatime,seclabel,data=ordered)
""".strip()

MOUNT_WITHOUT_ROOT = """
tmpfs on /tmp type tmpfs (rw,seclabel)
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
/dev/sda1 on /boot type ext4 (rw,relatime,seclabel,data=ordered)
""".strip()


PROC_MOUNT = """
proc /proc proc rw,relatime 0 0
sysfs /sys sysfs rw,relatime 0 0
devtmpfs /dev devtmpfs rw,relatime,size=8155456k,nr_inodes=2038864,mode=755 0 0
devpts /dev/pts devpts rw,relatime,gid=5,mode=620,ptmxmode=000 0 0
tmpfs /dev/shm tmpfs rw,nosuid,nodev,noexec,relatime 0 0
/dev/mapper/rootvg-rootlv / ext4 rw,relatime,barrier=1,data=ordered 0 0
/proc/bus/usb /proc/bus/usb usbfs rw,relatime 0 0
/dev/sda1 /boot ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-homelv /home ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-optlv /opt ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-tmplv /tmp ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-usrlv /usr ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-varlv /var ext4 rw,relatime,barrier=1,data=ordered 0 0
/dev/mapper/rootvg-tmplv /var/tmp ext4 rw,relatime,barrier=1,data=ordered 0 0
none /proc/sys/fs/binfmt_misc binfmt_misc rw,relatime 0 0
sunrpc /var/lib/nfs/rpc_pipefs rpc_pipefs rw,relatime 0 0
/vol/vol3/wpsanet /sa/net nfs rw,relatime,vers=3,rsize=8192,wsize=8192,namlen=255,soft,proto=tcp,timeo=14,retrans=2,sec=sys,mountaddr=10.xx.xx.xx,mountvers=3,mountport=4046,mountproto=udp,local_lock=none 0 0
/vol/vol9/home /users nfs rw,relatime,vers=3,rsize=8192,wsize=8192,namlen=255,soft,proto=tcp,timeo=14,retrans=2,sec=sys,mountaddr=10.xx.xx.xx,mountvers=3,mountport=4046,mountproto=udp,local_lock=none,addr=10.xx.xx.xx 0 0
/etc/auto.misc /misc autofs rw,relatime,fd=7,pgrp=1936,timeout=300,minproto=5,maxproto=5,indirect 0 0
""".strip()

PROC_MOUNT_QUOTES = """
/dev/mapper/rootvg-rootlv / ext4 rw,relatime,barrier=1,data=ordered 0 0
tmpfs /var/lib/containers/storage/overlay-containers/ff7e79fc09c/userdata/shm tmpfs rw,nosuid,nodev,noexec,relatime,context="system_u:object_r:container_file_t:s0:c184,c371",size=64000k 0 0
"""

PROCMOUNT_ERR_DATA = """
rootfs / rootfs rw 0 0
sysfs /sys sysfs rw,relatime
""".strip()

PROC_EXCEPTION1 = """
""".strip()

PROC_EXCEPTION2 = """
proc /proc proc rw,relatime 0 0
sysfs /sys sysfs rw,relatime 0 0
devtmpfs /dev devtmpfs rw,relatime,size=8155456k,nr_inodes=2038864,mode=755 0 0
""".strip()

MOUNTINFO_DATA = """
18 21 0:5 / /dev rw,relatime - devtmpfs devtmpfs rw,size=66002212k,nr_inodes=16500553,mode=755
19 18 0:11 / /dev/pts rw,relatime - devpts devpts rw,gid=5,mode=620,ptmxmode=000
20 18 0:16 / /dev/shm rw,nosuid,nodev,noexec,relatime - tmpfs tmpfs rw
21 1 253:1 / / rw,relatime - ext4 /dev/mapper/vgsys-root rw,barrier=1,stripe=64,data=ordered
23 21 8:1 / /boot rw,relatime - ext4 /dev/sda1 rw,barrier=1,stripe=64,data=ordered
25 21 253:32 / /diskdump rw,relatime - ext4 /dev/mapper/vgsys-diskdump rw,barrier=1,stripe=64,data=ordered
26 21 253:17 / /opt/CA rw,relatime - ext4 /dev/mapper/vgcore-opt_CA rw,barrier=1,stripe=64,data=ordered
36 21 253:34 / /var rw,relatime - ext4 /dev/mapper/vgsys-var rw,barrier=1,stripe=64,data=ordered
37 36 253:36 / /var/log rw,relatime - ext4 /dev/mapper/vgsys-var_log rw,barrier=1,stripe=64,data=ordered
38 36 253:22 / /var/opt/OV rw,relatime - ext4 /dev/mapper/vgcore-var_opt_OV rw,barrier=1,stripe=64,data=ordered
52 16 0:17 / /proc/sys/fs/binfmt_misc rw,relatime - binfmt_misc none rw
53 36 0:18 / /var/lib/nfs/rpc_pipefs rw,relatime - rpc_pipefs sunrpc rw
55 21 0:20 / /shared/dir1 rw,nosuid,relatime - nfs4 hostname1:/shared_dir1 rw,sync,vers=4,rsize=32768,wsize=32768,namlen=255,hard,proto=tcp,port=0,timeo=600,ret
56 21 0:19 / /shared/dir2 rw,nosuid,relatime - nfs hostname2:/shared/some/dir2 rw,sync,vers=3,rsize=32768,wsize=32
57 21 0:21 / /misc rw,relatime - autofs /etc/auto.misc rw,fd=7,pgrp=4826,timeout=300,minproto=5,maxproto=5,indirect
58 21 0:22 / /autofshost rw,relatime - autofs -hosts rw,fd=13,pgrp=4826,timeout=300,minproto=5,maxproto=5,indirect
""".strip()

MOUNTINFO_ERR_DATA = """
18 21 0:5 / /dev rw,relatime - devtmpfs devtmpfs rw,size=66002212k,nr_inodes=16500553,mode=755
20 18 0:16 / /dev/shm rw,nosuid,nodev,noexec,relatime - tmpfs tmpfs rw
21 1 253:1 / / rw,relatime  ext4 /dev/mapper/vgsys-root rw,barrier=1,stripe=64,data=ordered
23 21 8:1 / /boot rw,relatime - ext4 /dev/sda1 rw,barrier=1,stripe=64,data=ordered
""".strip()

MOUNTINFO_EXCEPTION = """
""".strip()

MOUNT_DOC = """
/dev/mapper/rootvg-rootlv on / type ext4 (rw,relatime,barrier=1,data=ordered)
proc on /proc type proc (rw,nosuid,nodev,noexec,relatime)
/dev/mapper/HostVG-Config on /etc/shadow type ext4 (rw,noatime,seclabel,stripe=256,data=ordered)
dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]
""".strip()

PROC_MOUNT_DOC = """
/dev/mapper/rootvg-rootlv / ext4 rw,relatime,barrier=1,data=ordered 0 0
proc /proc proc rw,nosuid,nodev,noexec,relatime 0 0
/dev/mapper/HostVG-Config /etc/shadow ext4 rw,noatime,seclabel,stripe=256,data=ordered 0 0
dev/sr0 /run/media/root/VMware\040Tools iso9660 ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2 0 0
""".strip()

MOUNTINFO_DOC = """
39 0 253:0 / / rw,relatime shared:1 - xfs /dev/mapper/rootvg-lvlocroot rw,attr2,inode64,noquota
47 39 8:1 / /boot rw,relatime shared:30 - xfs /dev/sda1 rw,attr2,inode64,noquota
65 39 253:19 / /data rw,relatime shared:44 - ext4 /dev/mapper/vgdata-lvdata rw,data=ordered
58 39 253:15 / /opt rw,relatime shared:45 - xfs /dev/mapper/rootvg-lvlocopt rw,attr2,inode64,noquota
61 39 253:17 / /home rw,nosuid,relatime shared:46 - xfs /dev/mapper/rootvg-lvlochome rw,attr2,inode64,noquotao
""".strip()

MOUNT_DATA_WITH_SPECIAL_MNT_POINT = """
hugetlbfs on /dev/hugepages type hugetlbfs (rw,relatime,seclabel)
/dev/sda1 on /boot type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/fedora-root on / type ext4 (rw,relatime,seclabel,data=ordered)
/dev/mapper/HostVG-Config on /etc/shadow type ext4 (rw,noatime,seclabel,stripe=256,data=ordered) [CONFIG]
/dev/sr0 on /run/media/root/VMware Tools type iso9660 (ro,nosuid,nodev,relatime,uid=0,gid=0,iocharset=utf8,mode=0400,dmode=0500,uhelper=udisks2) [VMware Tools]
/dev/sr1 on /run/media/Disk on C type NFS type iso9660 (ro,uid=0,gid=0,iocharset=utf8,uhelper=udisks2) [C type Disk]
""".strip()


def test_mount():
    results = Mount(context_wrap(MOUNT_DATA))
    assert results is not None
    assert len(results) == 13
    sr0 = results.search(filesystem='/dev/sr0')[0]
    sda1 = results.search(filesystem='/dev/sda1')[0]
    assert sr0 is not None
    assert sr0['mount_point'] == '/run/media/root/VMware Tools'
    # test get method
    assert sr0.get('mount_point') == '/run/media/root/VMware Tools'
    assert sr0.get('does not exist', 'failure') == 'failure'
    assert sr0['mount_type'] == 'iso9660'
    assert 'ro' in sr0['mount_options']
    assert sr0.mount_options.ro
    assert 'relatime' in sr0['mount_options']
    assert sr0['mount_options']['uhelper'] == 'udisks2'
    assert sr0['mount_label'] == '[VMware Tools]'
    assert sda1 is not None
    assert sda1['mount_point'] == '/boot'
    assert sda1['mount_type'] == 'ext4'
    assert 'rw' in sda1['mount_options']
    assert 'seclabel' in sda1['mount_options']
    assert sda1['mount_options']['data'] == 'ordered'
    assert sda1.mount_options.data == 'ordered'
    assert 'mount_label' not in sda1

    # Test iteration
    for mnt in results:
        assert hasattr(mnt, 'filesystem')
        assert hasattr(mnt, 'mount_point')
        assert hasattr(mnt, 'mount_type')
        assert hasattr(mnt, 'mount_options')

    # Test getitem
    assert results[12] == sr0
    assert results['/etc/shadow'] == results[11]
    # Index only by string or number
    with pytest.raises(TypeError) as exc:
        assert results[set([1, 2, 3])] is None
    assert "Mounts can only be indexed by mount string or line number" in str(exc)

    # Test mounts dictionary
    assert results.mounts['/run/media/root/VMware Tools'] == sr0

    # Test get_dir
    assert results.get_dir('/run/media/root/VMware Tools') == sr0
    assert results.get_dir('/boot/grub2/grub.cfg') == sda1
    assert results.get_dir('/etc') == results['/']
    assert results.get_dir('relative/paths/fail') is None

    # Test search
    assert results.search(filesystem='/dev/sr0') == [sr0]
    assert results.search(mount_type='tmpfs') == [
        results.rows[n] for n in (0, 7, 8)
    ]
    assert results.search(mount_options__contains='seclabel') == [
        results.rows[n] for n in (0, 1, 3, 4, 5, 7, 8, 11)
    ]


def test_mount_with_special_mnt_point():
    results = Mount(context_wrap(MOUNT_DATA_WITH_SPECIAL_MNT_POINT))
    assert len(results) == 6
    sr0 = results.search(filesystem='/dev/sr0')[0]
    sr1 = results.search(filesystem='/dev/sr1')[0]
    assert sr0['mount_point'] == '/run/media/root/VMware Tools'
    assert sr1['mount_point'] == '/run/media/Disk on C type NFS'
    assert sr0.get('mount_label') == '[VMware Tools]'
    assert sr1.get('mount_label') == '[C type Disk]'
    assert results['/run/media/Disk on C type NFS'].get('mount_label') == '[C type Disk]'


def test_mount_exception1():
    # Test parse failure
    with pytest.raises(ParseException) as pe:
        Mount(context_wrap(MOUNT_ERR_DATA))
    assert 'Unable to parse' in str(pe.value)


def test_mount_exception2():
    with pytest.raises(ParseException) as exc:
        Mount(context_wrap(MOUNT_WITHOUT_ROOT))
    assert "Input for mount must contain '/' mount point." in str(exc)


def test_proc_mount():
    results = ProcMounts(context_wrap(PROC_MOUNT))
    assert results is not None
    assert len(results) == 19
    sda1 = results.search(filesystem='/dev/sda1')[0]

    # Test get method
    assert sda1 is not None
    assert sda1['mount_point'] == '/boot'
    assert sda1['mount_type'] == 'ext4'
    assert 'rw' in sda1['mount_options']
    assert 'relatime' in sda1['mount_options']
    assert sda1['mount_options']['data'] == 'ordered'
    assert sda1.mount_options.data == 'ordered'
    assert sda1['mount_label'] == ['0', '0']

    # Test iteration
    for mnt in results:
        assert hasattr(mnt, 'filesystem')
        assert hasattr(mnt, 'mount_point')
        assert hasattr(mnt, 'mount_type')
        assert hasattr(mnt, 'mount_options')

    # Test getitem
    assert results[7] == sda1
    assert results['/misc'] == results[-1]
    # Index only by string or number
    with pytest.raises(TypeError) as exc:
        assert results[set([1, 2, 3])] is None
    assert 'Mounts can only be indexed by mount string or line number' in str(exc)

    # Test mounts dictionary
    assert results.mounts['/boot'] == sda1

    # Test get_dir
    assert results.get_dir('/var/lib/nfs/rpc_pipefs') == results.search(filesystem='sunrpc')[0]
    assert results.get_dir('/etc') == results['/']

    # Test search
    assert results.search(filesystem='/dev/sda1') == [sda1]
    assert results.search(mount_type='nfs') == [
        results.rows[n] for n in (16, 17)
    ]
    assert results.search(mount_options__contains='mode') == [
        results.rows[n] for n in (2, 3)
    ]


def test_proc_mount_quotes():
    results = ProcMounts(context_wrap(PROC_MOUNT_QUOTES))
    assert results is not None
    assert len(results) == 2
    device = results['/var/lib/containers/storage/overlay-containers/ff7e79fc09c/userdata/shm']
    assert device.mount_options.context == "system_u:object_r:container_file_t:s0:c184,c371"


def test_proc_mount_exception1():
    with pytest.raises(SkipComponent) as e:
        ProcMounts(context_wrap(PROC_EXCEPTION1))
    assert 'Empty content' in str(e)


def test_proc_mount_exception2():
    with pytest.raises(ParseException) as e:
        ProcMounts(context_wrap(PROC_EXCEPTION2))
    assert "Input for mount must contain '/' mount point." in str(e)


def test_proc_mount_exception3():
    with pytest.raises(ParseException) as pe:
        ProcMounts(context_wrap(PROCMOUNT_ERR_DATA))
    assert 'Unable to parse' in str(pe.value)


def test_mountinfo():
    results = MountInfo(context_wrap(MOUNTINFO_DATA))
    assert results is not None
    assert len(results) == 16

    sda1 = results.search(mount_source='/dev/sda1')[0]
    assert sda1 is not None
    assert sda1['mount_point'] == '/boot'
    # test get method
    assert sda1.get('mount_point') == '/boot'
    assert sda1.get('does_not_exist', 'failure') == 'failure'
    assert sda1['mount_type'] == 'ext4'
    assert 'rw' in sda1['mount_options']
    assert sda1.mount_options.rw
    assert 'data' in sda1['mount_options']
    assert sda1['mount_options']['data'] == 'ordered'
    assert 'mount_label' not in sda1
    assert sda1 is not None

    dir1 = results.search(mount_point='/shared/dir1')[0]
    assert dir1['mount_source'] == 'hostname1:/shared_dir1'
    assert dir1['mount_type'] == 'nfs4'
    assert 'vers' in dir1['mount_options']
    assert dir1['mount_options']['vers'] == '4'
    assert dir1['mount_addtlinfo'].major_minor == '0:20'
    assert dir1.mount_addtlinfo.mount_id == '55'

    # Test iteration
    for mnt in results:
        assert hasattr(mnt, 'mount_source')
        assert hasattr(mnt, 'mount_point')
        assert hasattr(mnt, 'mount_type')
        assert hasattr(mnt, 'mount_options')

    # Test getitem
    assert results[4] == sda1
    assert results['/shared/dir1'] == results[12]
    # Index only by string or number
    with pytest.raises(TypeError) as exc:
        assert results[set([1, 2, 3])] is None
    assert "Mounts can only be indexed by mount string or line number" in str(exc)

    # Test mounts dictionary
    assert results.mounts['/boot'] == sda1

    # Test get_dir
    assert results.get_dir('/var/log/some_dir') == results.search(mount_point='/var/log')[0]
    assert results.get_dir('/etc') == results['/']

    # Test search
    assert results.search(mount_type='nfs') == [results.rows[13]]
    assert results.search(mount_options__contains='mode') == [
        results.rows[n] for n in (0, 1)
    ]


def test_doc_examples():
    env = {
            'mnt_info': Mount(context_wrap(MOUNT_DOC)),
            'proc_mnt_info': ProcMounts(context_wrap(PROC_MOUNT_DOC)),
            'proc_mountinfo': MountInfo(context_wrap(MOUNTINFO_DOC)),
            'mounts': Mount(context_wrap(MOUNT_DOC))

          }
    failed, total = doctest.testmod(mount, globs=env)
    assert failed == 0



import pytest
from insights import SkipComponent
from insights.tests import context_wrap
from insights.parsers.ps import PsAuxcww
from insights.combiners.ps import Ps
from insights.components.ceph import IsCephMonitor

PsAuxcww_CEPH = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 init
user2    20164  0.0  0.0 108472  1896 pts/5    Ss   10:10   0:00 bash
root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 dhclient
root     22673  0.6 10.7 1618556 840452 ?      Sl   11:38   1:31 ceph-mon
vdsm     27323 98.0 11.3  9120    987 ?        Ss   10.01   1:31 vdsm
""".strip()

PsAuxcww_NO_CEPH = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 init
user2    20164  0.0  0.0 108472  1896 pts/5    Ss   10:10   0:00 bash
root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 dhclient
vdsm     27323 98.0 11.3  9120    987 ?        Ss   10.01   1:31 vdsm
""".strip()


def test_is_ceph_monitor():
    ps_auxcww = PsAuxcww(context_wrap(PsAuxcww_CEPH))
    ps = Ps(None, None, None, None, ps_auxcww, None, None)
    result = IsCephMonitor(ps)
    assert isinstance(result, IsCephMonitor)

    ps_auxcww = PsAuxcww(context_wrap(PsAuxcww_NO_CEPH))
    ps = Ps(None, None, None, None, ps_auxcww, None, None)
    with pytest.raises(SkipComponent):
        IsCephMonitor(ps)



import pytest

from insights.core.exceptions import SkipComponent
from insights.specs.datasources.mount import xfs_mounts
from insights.parsers.mount import ProcMounts
from insights.tests import context_wrap

PROC_MOUNT_XFS = '''
/dev/mapper/rhel_insights--release-root / xfs rw,seclabel,relatime,attr2,inode64,logbufs=8,logbsize=32k,noquota 0 0
/dev/vda1 /boot xfs rw,seclabel,relatime,attr2,inode64,logbufs=8,logbsize=32k,noquota 0 0
'''.strip()

PROC_MOUNT_NO_XFS = '''
/dev/mapper/rhel_insights--release-root / ext4 rw,seclabel,relatime,attr2,inode64,logbufs=8,logbsize=32k,noquota 0 0
fusectl /sys/fs/fuse/connections fusectl rw,relatime 0 0
tmpfs /run/user/1000 tmpfs rw,seclabel,nosuid,nodev,relatime,size=786664k,mode=700,uid=1000,gid=10 0 0
binfmt_misc /proc/sys/fs/binfmt_misc binfmt_misc rw,relatime 0 0
'''.strip()


def test_xfs_mounts():
    pmnt = ProcMounts(context_wrap(PROC_MOUNT_XFS))
    broker = {ProcMounts: pmnt}
    result = xfs_mounts(broker)
    assert result == ['/', '/boot']

    pmnt = ProcMounts(context_wrap(PROC_MOUNT_NO_XFS))
    broker = {ProcMounts: pmnt}
    with pytest.raises(SkipComponent):
        xfs_mounts(broker)
